# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

""""An object that manages the buffering and delaying of observation."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

import numpy as np
import six
from six.moves import range


class InFlightObservation(object):
  """Represents a delayed observation that may not have arrived yet.

  Attributes:
    arrival: The time at which this observation will be delivered.
    timestamp: The time at which this observation was made.
    delay: The amount of delay between the time at which this observation was
      made and the time at which it is delivered.
    value: The value of this observation.
  """

  __slots__ = ('arrival', 'timestamp', 'delay', 'value')

  def __init__(self, timestamp, delay, value):
    self.arrival = timestamp + delay
    self.timestamp = timestamp
    self.delay = delay
    self.value = value

  def __lt__(self, other):
    # This is implemented to facilitate sorting.
    return self.arrival < other.arrival


class Buffer(object):
  """An object that manages the buffering and delaying of observation."""

  def __init__(self, buffer_size, shape, dtype, pad_value=0,
               strip_singleton_buffer_dim=False):
    """Initializes this observation buffer.

    Args:
      buffer_size: The size of the buffer returned by `read`. Note
        that this does *not* affect size of the internal buffer held by this
        object, which always grow as large as is necessary in the presence of
        large delays.
      shape: The shape of a single observation held by this buffer, which can
        either be a single integer or an iterable of integers. The shape of the
        buffer returned by `read` will then be
        `(buffer_size, shape[0], ..., shape[n])`, unless `buffer_size == 1`
        and `strip_singleton_buffer_dim == True`.
      dtype: The NumPy dtype of observation entries.
      pad_value: (optional) The value that is used to pad the buffer returned
        by `read` when the number of observation entries is less
        then `buffer_size`. Specifically, the buffer will be padded by
        `np.full(shape, pad_value, dtype)`.
      strip_singleton_buffer_dim: (optional) A boolean, if `True` and
        `buffer_size == 1` then the leading dimension will not be added to the
        shape of the array returned by `read`.
    """
    self._buffer_size = buffer_size
    try:
      shape = tuple(shape)
    except TypeError:
      if isinstance(shape, int):
        shape = (shape,)
      else:
        raise

    self._has_buffer_dim = not (strip_singleton_buffer_dim and buffer_size == 1)
    if self._has_buffer_dim:
      self._buffered_shape = (buffer_size,) + shape
    else:
      self._buffered_shape = shape
    self._dtype = dtype

    # The "arrived" deque contains entries that are due to be delivered now.
    # This deque should never grow beyond buffer_size.
    self._arrived_deque = collections.deque(maxlen=buffer_size)
    for _ in range(buffer_size):
      self._arrived_deque.append(
          InFlightObservation(-np.inf, 0, np.full(shape, pad_value, dtype)))

    # The "pending" deque contains entries that are stored for future delivery.
    # This deque can grow arbitrarily large in presence of long delays.
    self._pending_deque = collections.deque()

  def _update_arrived_deque(self, timestamp):
    while self._pending_deque and self._pending_deque[0].arrival <= timestamp:
      self._arrived_deque.append(self._pending_deque.popleft())

  @property
  def shape(self):
    return self._buffered_shape

  @property
  def dtype(self):
    return self._dtype

  def insert(self, timestamp, delay, value):
    """Inserts a new observation to the buffer.

    This function implicitly updates the internal "clock" of this buffer to
    the timestamp of the new observation, and the internal buffer is trimmed
    accordingly, i.e. at most `buffer_size` items whose delayed arrival time
    preceeds `timestamp` are kept.

    Args:
      timestamp: The time at which this observation was made.
      delay: The amount of delay between the time at which this observation was
        made and the time at which it is delivered.
      value: The value of this observation.

    Raises:
      ValueError: if `delay` is negative.
    """
    self._update_arrived_deque(timestamp)
    new_obs = InFlightObservation(timestamp, delay, np.array(value))
    arrival = new_obs.arrival
    if delay == 0:
      # No delay, so the new observation is due for immediate delivery.
      # Add it to the arrived deque.
      self._arrived_deque.append(new_obs)
    elif delay > 0:
      if not self._pending_deque or arrival > self._pending_deque[-1].arrival:
        # New observation's arrival time is monotonic.
        # Technically, we can handle this in the general code branch below,
        # but since this is assumed to be the "typical" case, the special
        # handling here saves us from repeatedly allocating and deallocating
        # an empty temporary deque.
        self._pending_deque.append(new_obs)
      else:
        # General, out-of-order observation.
        arriving_after_new_obs = collections.deque()
        while self._pending_deque and arrival < self._pending_deque[-1].arrival:
          arriving_after_new_obs.appendleft(self._pending_deque.pop())
        self._pending_deque.append(new_obs)
        for existing_obs in arriving_after_new_obs:
          self._pending_deque.append(existing_obs)
    else:
      raise ValueError('`delay` should not be negative: '
                       'got {!r}'.format(delay))

  def read(self, current_time):
    """Reads the content of the buffer at the given timestamp."""
    self._update_arrived_deque(current_time)
    if self._has_buffer_dim:
      out = np.empty(self._buffered_shape, dtype=self._dtype)
      for i, obs in enumerate(self._arrived_deque):
        out[i] = obs.value
    else:
      out = self._arrived_deque[0].value.copy()
    return out

  def drop_unobserved_upcoming_items(self, observation_schedule, read_interval):
    """Plans an optimal observation schedule for an upcoming control period.

    This function determines which of the proposed upcoming observations will
    never in fact be delivered and removes them from the observation schedule.

    We assume that observations will only be queried at times that are integer
    multiples of `read_interval`. If more observations are generated during
    the upcoming control step than the `buffer_size` of this `Buffer`
    then of those new observations will never be required. This function takes
    into account the delayed arrival time and existing buffered items in the
    planning process.

    Args:
      observation_schedule: An list of `(timestamp, delay)` tuples, where
        `timestamp` is the time at which the observation value will be produced,
        and `delay` is the amount of time the observation will be delayed by.
        This list will be modified in place.
      read_interval: The time interval between successive calls to `read`.
        We assume that observations will only be queried at times that are
        integer multiples of `read_interval`.
    """
    # Private deques to simulate what the deques will look like in the future,
    # according to the proposed upcoming observation schedule.
    future_arrived_deque = collections.deque()
    future_pending_deque = collections.deque()

    # Take existing buffered observations into account when planning the
    # upcoming schedule.
    def get_next_existing_timestamp():
      for obs in reversed(self._pending_deque):
        yield InFlightObservation(obs.timestamp, obs.delay, None)
      while True:
        yield InFlightObservation(-np.inf, 0, None)
    existing_timestamp_iter = get_next_existing_timestamp()
    existing_timestamp = six.next(existing_timestamp_iter)

    # Build the simulated state of the pending deque at the end of the proposed
    # schedule.
    sorted_schedule = sorted([InFlightObservation(time[0], time[1], None)
                              for time in observation_schedule])
    for new_timestamp in reversed(sorted_schedule):
      # We don't need to worry about any existing item that are delivered before
      # the first new item, since those are purged independently of our
      # proposed new observations.
      while existing_timestamp.arrival > new_timestamp.arrival:
        future_pending_deque.appendleft(existing_timestamp)
        existing_timestamp = six.next(existing_timestamp_iter)
      future_pending_deque.appendleft(new_timestamp)

    # Find the next timestep at which `read` is called.
    first_proposed_timestamp = min(t for t, _ in observation_schedule)
    next_read_time = read_interval * int(np.ceil(
        first_proposed_timestamp // read_interval))

    # Build the simulated state of the arrived deque at each subsequent
    # control steps.
    while future_pending_deque:
      # Keep track of observations that are delivered for the first time
      # during this control timestep.
      newly_arrived = collections.deque()
      while (future_pending_deque and
             future_pending_deque[0].arrival <= next_read_time):
        # `fake_observation` is an `InFlightObservation` without `value`.
        fake_observation = future_pending_deque.popleft()
        future_arrived_deque.append(fake_observation)
        newly_arrived.append(fake_observation)
      while len(future_arrived_deque) > self._buffer_size:
        stale = future_arrived_deque.popleft()
        # Newly-arrived items that become immediately stale are never actually
        # delivered.
        if newly_arrived and stale == newly_arrived[0]:
          newly_arrived.popleft()
          # `stale` might either be one of the existing pending observations or
          # from the proposed schedule.
          if stale.timestamp >= first_proposed_timestamp:
            observation_schedule.remove((stale.timestamp, stale.delay))

      next_read_time += read_interval



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Utilities for testing rendering."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections
import functools
import os
import sys
from dm_control import _render
from dm_control import mujoco
from dm_control.mujoco.testing import assets
import numpy as np
from PIL import Image
import six
from six.moves import range
from six.moves import zip


BACKEND_STRING = 'hardware' if _render.USING_GPU else 'software'


class ImagesNotCloseError(AssertionError):
  """Exception raised when two images are not sufficiently close."""

  def __init__(self, message, expected, actual):
    super(ImagesNotCloseError, self).__init__(message)
    self.expected = expected
    self.actual = actual


_CameraSpec = collections.namedtuple(
    '_CameraSpec', ['height', 'width', 'camera_id', 'render_flag_overrides'])


_SUBDIR_TEMPLATE = (
    '{name}_seed_{seed}_camera_{camera_id}_{width}x{height}_{backend_string}'
    '{render_flag_overrides_string}'
)


def _get_subdir(name, seed, backend_string, camera_spec):
  if camera_spec.render_flag_overrides:
    overrides = ('{}_{}'.format(k, v) for k, v in
                 sorted(camera_spec.render_flag_overrides.items()))
    render_flag_overrides_string = '_' + '_'.join(overrides)
  else:
    render_flag_overrides_string = ''
  return _SUBDIR_TEMPLATE.format(
      name=name,
      seed=seed,
      camera_id=camera_spec.camera_id,
      width=camera_spec.width,
      height=camera_spec.height,
      backend_string=backend_string,
      render_flag_overrides_string=render_flag_overrides_string,
  )


class _FrameSequence(object):
  """A sequence of pre-rendered frames used in integration tests."""

  _ASSETS_DIR = 'assets'
  _FRAMES_DIR = 'frames'
  _FILENAME_TEMPLATE = 'frame_{frame_num:03}.png'

  def __init__(self,
               name,
               xml_string,
               camera_specs,
               num_frames=20,
               steps_per_frame=10,
               seed=0):
    """Initializes a new `_FrameSequence`.

    Args:
      name: A string containing the name to be used for the sequence.
      xml_string: An MJCF XML string containing the model to be rendered.
      camera_specs: A list of `_CameraSpec` instances specifying the cameras to
        render on each frame.
      num_frames: The number of frames to render.
      steps_per_frame: The interval between frames, in simulation steps.
      seed: Integer or None, used to initialize the random number generator for
        generating actions.
    """
    self._name = name
    self._xml_string = xml_string
    self._camera_specs = camera_specs
    self._num_frames = num_frames
    self._steps_per_frame = steps_per_frame
    self._seed = seed

  @property
  def num_cameras(self):
    return len(self._camera_specs)

  def iter_render(self):
    """Returns an iterator that yields newly rendered frames as numpy arrays."""
    random_state = np.random.RandomState(self._seed)
    physics = mujoco.Physics.from_xml_string(self._xml_string)
    action_spec = mujoco.action_spec(physics)
    for _ in range(self._num_frames):
      for _ in range(self._steps_per_frame):
        actions = random_state.uniform(action_spec.minimum, action_spec.maximum)
        physics.set_control(actions)
        physics.step()
      for camera_spec in self._camera_specs:
        yield physics.render(**camera_spec._asdict())

  def iter_load(self):
    """Returns an iterator that yields saved frames as numpy arrays."""
    for directory, filename in self._iter_paths():
      path = os.path.join(directory, filename)
      yield _load_pixels(path)

  def save(self):
    """Saves a new set of golden output frames to disk."""
    for pixels, (relative_to_assets, filename) in zip(self.iter_render(),
                                                      self._iter_paths()):
      full_directory_path = os.path.join(self._ASSETS_DIR, relative_to_assets)
      if not os.path.exists(full_directory_path):
        os.makedirs(full_directory_path)
      path = os.path.join(full_directory_path, filename)
      _save_pixels(pixels, path)

  def _iter_paths(self):
    """Returns an iterator over paths to the reference images."""
    for frame_num in range(self._num_frames):
      filename = self._FILENAME_TEMPLATE.format(frame_num=frame_num)
      for camera_spec in self._camera_specs:
        subdir_name = _get_subdir(
            name=self._name,
            seed=self._seed,
            backend_string=BACKEND_STRING,
            camera_spec=camera_spec)
        directory = os.path.join(self._FRAMES_DIR, subdir_name)
        yield directory, filename


cartpole = _FrameSequence(
    name='cartpole',
    xml_string=assets.get_contents('cartpole.xml'),
    camera_specs=(
        _CameraSpec(
            width=320, height=240, camera_id=0, render_flag_overrides={}),
    ),
    steps_per_frame=5)

humanoid = _FrameSequence(
    name='humanoid',
    xml_string=assets.get_contents('humanoid.xml'),
    camera_specs=(
        _CameraSpec(
            width=240, height=320, camera_id=0, render_flag_overrides={}),
        _CameraSpec(
            width=240,
            height=320,
            camera_id=0,
            render_flag_overrides={
                'shadow': False,
                'reflection': False,
            }),
        _CameraSpec(
            width=64,
            height=64,
            camera_id='head_track',
            render_flag_overrides={}),
    ))


SEQUENCES = {
    'cartpole': cartpole,
    'humanoid': humanoid,
}


def _save_pixels(pixels, path):
  image = Image.fromarray(pixels)
  image.save(path)


def _load_pixels(path):
  image_bytes = assets.get_contents(path)
  image = Image.open(six.BytesIO(image_bytes))
  return np.array(image)


def compute_rms(image1, image2):
  """Computes the RMS difference between two images."""
  abs_diff = np.abs(image1.astype(np.int16) - image2)
  values, counts = np.unique(abs_diff, return_counts=True)
  sum_of_squares = np.sum(counts * values.astype(np.int64) ** 2)
  return np.sqrt(float(sum_of_squares) / abs_diff.size)


def assert_images_close(expected, actual, tolerance=10.):
  """Tests whether two images are almost equal.

  Args:
    expected: A numpy array, the expected image.
    actual: A numpy array, the actual image.
    tolerance: A float specifying the maximum allowable RMS error between the
      expected and actual images.

  Raises:
    ImagesNotCloseError: If the images are not sufficiently close.
  """
  rms = compute_rms(expected, actual)
  if rms > tolerance:
    message = 'RMS error exceeds tolerance ({} > {})'.format(rms, tolerance)
    raise ImagesNotCloseError(message, expected=expected, actual=actual)


def save_images_on_failure(output_dir):
  """Decorator that saves debugging images if `ImagesNotCloseError` is raised.

  Args:
    output_dir: Path to the directory where the output images will be saved.

  Returns:
    A decorator function.
  """
  def decorator(test_method):
    """Decorator, saves debugging images if `ImagesNotCloseError` is raised."""
    method_name = test_method.__name__
    @functools.wraps(test_method)
    def decorated_method(*args, **kwargs):
      """Call test method, save images if `ImagesNotCloseError` is raised."""
      try:
        test_method(*args, **kwargs)
      except ImagesNotCloseError as e:
        _, _, tb = sys.exc_info()
        if not os.path.exists(output_dir):
          os.makedirs(output_dir)
        difference = e.actual.astype(np.double) - e.expected
        difference = (0.5 * (difference + 255)).astype(np.uint8)
        base_name = os.path.join(output_dir, method_name)
        _save_pixels(e.expected, base_name + '-expected.png')
        _save_pixels(e.actual, base_name + '-actual.png')
        _save_pixels(difference, base_name + '-difference.png')
        msg = ('{}. Debugging images saved to '
               '{}-{{expected,actual,difference}}.png.'.format(e, base_name))
        new_e = ImagesNotCloseError(msg, expected=e.expected, actual=e.actual)
        # Reraise the exception with the original traceback.
        six.reraise(ImagesNotCloseError, new_e, tb)

    return decorated_method
  return decorator



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""OpenGL utility for rendering numpy arrays as images on a quad surface."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import ctypes

import numpy as np
from OpenGL import GL
from OpenGL.GL import shaders

# This array contains packed position and texture cooridnates of a fullscreen
# quad.
# It contains definition of 4 vertices that will be rendered as a triangle
# strip. Each vertex is described by a tuple:
# (VertexPosition.X, VertexPosition.Y, TextureCoord.U, TextureCoord.V)
_FULLSCREEN_QUAD_VERTEX_POSITONS_AND_TEXTURE_COORDS = np.array([
    -1, -1, 0, 1,
    -1, 1, 0, 0,
    1, -1, 1, 1,
    1, 1, 1, 0], dtype=np.float32)
_FLOATS_PER_XY = 2
_FLOATS_PER_VERTEX = 4
_SIZE_OF_FLOAT = ctypes.sizeof(ctypes.c_float)

_VERTEX_SHADER = """
#version 120
attribute vec2 position;
attribute vec2 uv;
void main() {
  gl_Position = vec4(position, 0, 1);
  gl_TexCoord[0].st = uv;
}
"""
_FRAGMENT_SHADER = """
#version 120
uniform sampler2D tex;
void main() {
  gl_FragColor = texture2D(tex, gl_TexCoord[0].st);
}
"""
_VAR_POSITION = 'position'
_VAR_UV = 'uv'
_VAR_TEXTURE_SAMPLER = 'tex'


class FullscreenQuadRenderer(object):
  """Renders pixmaps on a fullscreen quad using OpenGL."""

  def __init__(self):
    """Initializes the fullscreen quad renderer."""
    GL.glClearColor(0, 0, 0, 0)
    self._init_geometry()
    self._init_texture()
    self._init_shaders()

  def _init_geometry(self):
    """Initializes the fullscreen quad geometry."""
    vertex_buffer = GL.glGenBuffers(1)
    GL.glBindBuffer(GL.GL_ARRAY_BUFFER, vertex_buffer)
    GL.glBufferData(
        GL.GL_ARRAY_BUFFER,
        _FULLSCREEN_QUAD_VERTEX_POSITONS_AND_TEXTURE_COORDS.nbytes,
        _FULLSCREEN_QUAD_VERTEX_POSITONS_AND_TEXTURE_COORDS, GL.GL_STATIC_DRAW)

  def _init_texture(self):
    """Initializes the texture storage."""
    self._texture = GL.glGenTextures(1)
    GL.glBindTexture(GL.GL_TEXTURE_2D, self._texture)
    GL.glTexParameteri(
        GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
    GL.glTexParameteri(
        GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)

  def _init_shaders(self):
    """Initializes the shaders used to render the textures fullscreen quad."""
    vs = shaders.compileShader(_VERTEX_SHADER, GL.GL_VERTEX_SHADER)
    fs = shaders.compileShader(_FRAGMENT_SHADER, GL.GL_FRAGMENT_SHADER)
    self._shader = shaders.compileProgram(vs, fs)

    stride = _FLOATS_PER_VERTEX * _SIZE_OF_FLOAT
    var_position = GL.glGetAttribLocation(self._shader, _VAR_POSITION)
    GL.glVertexAttribPointer(
        var_position, 2, GL.GL_FLOAT, GL.GL_FALSE, stride, None)
    GL.glEnableVertexAttribArray(var_position)

    var_uv = GL.glGetAttribLocation(self._shader, _VAR_UV)
    uv_offset = ctypes.c_void_p(_FLOATS_PER_XY * _SIZE_OF_FLOAT)
    GL.glVertexAttribPointer(
        var_uv, 2, GL.GL_FLOAT, GL.GL_FALSE, stride, uv_offset)
    GL.glEnableVertexAttribArray(var_uv)

    self._var_texture_sampler = GL.glGetUniformLocation(
        self._shader, _VAR_TEXTURE_SAMPLER)

  def render(self, pixmap, viewport_shape):
    """Renders the pixmap on a fullscreen quad.

    Args:
      pixmap: A 3D numpy array of bytes (np.uint8), with dimensions
        (width, height, 3).
      viewport_shape: A tuple of two elements, (width, height).
    """
    GL.glClear(GL.GL_COLOR_BUFFER_BIT)
    GL.glViewport(0, 0, *viewport_shape)
    GL.glUseProgram(self._shader)
    GL.glActiveTexture(GL.GL_TEXTURE0)
    GL.glBindTexture(GL.GL_TEXTURE_2D, self._texture)
    GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT, 1)
    GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, pixmap.shape[1],
                    pixmap.shape[0], 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE,
                    pixmap)
    GL.glUniform1i(self._var_texture_sampler, 0)
    GL.glDrawArrays(GL.GL_TRIANGLE_STRIP, 0, 4)



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Procedurally generated LQR domain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections
import os

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.utils import containers
from dm_control.utils import xml_tools
from lxml import etree
import numpy as np
from six.moves import range

from dm_control.utils import io as resources

_DEFAULT_TIME_LIMIT = float('inf')
_CONTROL_COST_COEF = 0.1
SUITE = containers.TaggedTasks()


def get_model_and_assets(n_bodies, n_actuators, random):
  """Returns the model description as an XML string and a dict of assets.

  Args:
    n_bodies: An int, number of bodies of the LQR.
    n_actuators: An int, number of actuated bodies of the LQR. `n_actuators`
      should be less or equal than `n_bodies`.
    random: A `numpy.random.RandomState` instance.

  Returns:
    A tuple `(model_xml_string, assets)`, where `assets` is a dict consisting of
    `{filename: contents_string}` pairs.
  """
  return _make_model(n_bodies, n_actuators, random), common.ASSETS


@SUITE.add()
def lqr_2_1(time_limit=_DEFAULT_TIME_LIMIT, random=None,
            environment_kwargs=None):
  """Returns an LQR environment with 2 bodies of which the first is actuated."""
  return _make_lqr(n_bodies=2,
                   n_actuators=1,
                   control_cost_coef=_CONTROL_COST_COEF,
                   time_limit=time_limit,
                   random=random,
                   environment_kwargs=environment_kwargs)


@SUITE.add()
def lqr_6_2(time_limit=_DEFAULT_TIME_LIMIT, random=None,
            environment_kwargs=None):
  """Returns an LQR environment with 6 bodies of which first 2 are actuated."""
  return _make_lqr(n_bodies=6,
                   n_actuators=2,
                   control_cost_coef=_CONTROL_COST_COEF,
                   time_limit=time_limit,
                   random=random,
                   environment_kwargs=environment_kwargs)


def _make_lqr(n_bodies, n_actuators, control_cost_coef, time_limit, random,
              environment_kwargs):
  """Returns a LQR environment.

  Args:
    n_bodies: An int, number of bodies of the LQR.
    n_actuators: An int, number of actuated bodies of the LQR. `n_actuators`
      should be less or equal than `n_bodies`.
    control_cost_coef: A number, the coefficient of the control cost.
    time_limit: An int, maximum time for each episode in seconds.
    random: Either an existing `numpy.random.RandomState` instance, an
      integer seed for creating a new `RandomState`, or None to select a seed
      automatically.
    environment_kwargs: A `dict` specifying keyword arguments for the
      environment, or None.

  Returns:
    A LQR environment with `n_bodies` bodies of which first `n_actuators` are
    actuated.
  """

  if not isinstance(random, np.random.RandomState):
    random = np.random.RandomState(random)

  model_string, assets = get_model_and_assets(n_bodies, n_actuators,
                                              random=random)
  physics = Physics.from_xml_string(model_string, assets=assets)
  task = LQRLevel(control_cost_coef, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(physics, task, time_limit=time_limit,
                             **environment_kwargs)


def _make_body(body_id, stiffness_range, damping_range, random):
  """Returns an `etree.Element` defining a body.

  Args:
    body_id: Id of the created body.
    stiffness_range: A tuple of (stiffness_lower_bound, stiffness_uppder_bound).
      The stiffness of the joint is drawn uniformly from this range.
    damping_range: A tuple of (damping_lower_bound, damping_upper_bound). The
      damping of the joint is drawn uniformly from this range.
    random: A `numpy.random.RandomState` instance.

  Returns:
   A new instance of `etree.Element`. A body element with two children: joint
   and geom.
  """
  body_name = 'body_{}'.format(body_id)
  joint_name = 'joint_{}'.format(body_id)
  geom_name = 'geom_{}'.format(body_id)

  body = etree.Element('body', name=body_name)
  body.set('pos', '.25 0 0')
  joint = etree.SubElement(body, 'joint', name=joint_name)
  body.append(etree.Element('geom', name=geom_name))
  joint.set('stiffness',
            str(random.uniform(stiffness_range[0], stiffness_range[1])))
  joint.set('damping',
            str(random.uniform(damping_range[0], damping_range[1])))
  return body


def _make_model(n_bodies,
                n_actuators,
                random,
                stiffness_range=(15, 25),
                damping_range=(0, 0)):
  """Returns an MJCF XML string defining a model of springs and dampers.

  Args:
    n_bodies: An integer, the number of bodies (DoFs) in the system.
    n_actuators: An integer, the number of actuated bodies.
    random: A `numpy.random.RandomState` instance.
    stiffness_range: A tuple containing minimum and maximum stiffness. Each
      joint's stiffness is sampled uniformly from this interval.
    damping_range: A tuple containing minimum and maximum damping. Each joint's
      damping is sampled uniformly from this interval.

  Returns:
    An MJCF string describing the linear system.

  Raises:
    ValueError: If the number of bodies or actuators is erronous.
  """
  if n_bodies < 1 or n_actuators < 1:
    raise ValueError('At least 1 body and 1 actuator required.')
  if n_actuators > n_bodies:
    raise ValueError('At most 1 actuator per body.')

  file_path = os.path.join(os.path.dirname(__file__), 'lqr.xml')
  with resources.GetResourceAsFile(file_path) as xml_file:
    mjcf = xml_tools.parse(xml_file)
  parent = mjcf.find('./worldbody')
  actuator = etree.SubElement(mjcf.getroot(), 'actuator')
  tendon = etree.SubElement(mjcf.getroot(), 'tendon')

  for body in range(n_bodies):
    # Inserting body.
    child = _make_body(body, stiffness_range, damping_range, random)
    site_name = 'site_{}'.format(body)
    child.append(etree.Element('site', name=site_name))

    if body == 0:
      child.set('pos', '.25 0 .1')
    # Add actuators to the first n_actuators bodies.
    if body < n_actuators:
      # Adding actuator.
      joint_name = 'joint_{}'.format(body)
      motor_name = 'motor_{}'.format(body)
      child.find('joint').set('name', joint_name)
      actuator.append(etree.Element('motor', name=motor_name, joint=joint_name))

    # Add a tendon between consecutive bodies (for visualisation purposes only).
    if body < n_bodies - 1:
      child_site_name = 'site_{}'.format(body + 1)
      tendon_name = 'tendon_{}'.format(body)
      spatial = etree.SubElement(tendon, 'spatial', name=tendon_name)
      spatial.append(etree.Element('site', site=site_name))
      spatial.append(etree.Element('site', site=child_site_name))
    parent.append(child)
    parent = child

  return etree.tostring(mjcf, pretty_print=True)


class Physics(mujoco.Physics):
  """Physics simulation with additional features for the LQR domain."""

  def state_norm(self):
    """Returns the norm of the physics state."""
    return np.linalg.norm(self.state())


class LQRLevel(base.Task):
  """A Linear Quadratic Regulator `Task`."""

  _TERMINAL_TOL = 1e-6

  def __init__(self, control_cost_coef, random=None):
    """Initializes an LQR level with cost = sum(states^2) + c*sum(controls^2).

    Args:
      control_cost_coef: The coefficient of the control cost.
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).

    Raises:
      ValueError: If the control cost coefficient is not positive.
    """
    if control_cost_coef <= 0:
      raise ValueError('control_cost_coef must be positive.')

    self._control_cost_coef = control_cost_coef
    super(LQRLevel, self).__init__(random=random)

  @property
  def control_cost_coef(self):
    return self._control_cost_coef

  def initialize_episode(self, physics):
    """Random state sampled from a unit sphere."""
    ndof = physics.model.nq
    unit = self.random.randn(ndof)
    physics.data.qpos[:] = np.sqrt(2) * unit / np.linalg.norm(unit)
    super(LQRLevel, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns an observation of the state."""
    obs = collections.OrderedDict()
    obs['position'] = physics.position()
    obs['velocity'] = physics.velocity()
    return obs

  def get_reward(self, physics):
    """Returns a quadratic state and control reward."""
    position = physics.position()
    state_cost = 0.5 * np.dot(position, position)
    control_signal = physics.control()
    control_l2_norm = 0.5 * np.dot(control_signal, control_signal)
    return 1 - (state_cost + control_l2_norm * self._control_cost_coef)

  def get_evaluation(self, physics):
    """Returns a sparse evaluation reward that is not used for learning."""
    return float(physics.state_norm() <= 0.01)

  def get_termination(self, physics):
    """Terminates when the state norm is smaller than epsilon."""
    if physics.state_norm() < self._TERMINAL_TOL:
      return 0.0



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for dm_control.locomotion.soccer.load."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from absl import logging
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.locomotion import soccer
import numpy as np
from six.moves import range


class LoadTest(parameterized.TestCase):

  @parameterized.named_parameters(
      ("2vs2_nocontacts", 2, True), ("2vs2_contacts", 2, False),
      ("1vs1_nocontacts", 1, True), ("1vs1_contacts", 1, False))
  def test_load_env(self, team_size, disable_walker_contacts):
    env = soccer.load(team_size=team_size, time_limit=2.,
                      disable_walker_contacts=disable_walker_contacts)
    action_specs = env.action_spec()

    random_state = np.random.RandomState(0)
    time_step = env.reset()
    while not time_step.last():
      actions = []
      for action_spec in action_specs:
        action = random_state.uniform(
            action_spec.minimum, action_spec.maximum, size=action_spec.shape)
        actions.append(action)
      time_step = env.step(actions)

      for i in range(len(action_specs)):
        logging.info(
            "Player %d: reward = %s, discount = %s, observations = %s.", i,
            time_step.reward[i], time_step.discount, time_step.observation[i])

  def assertSameObservation(self, expected_observation, actual_observation):
    self.assertLen(actual_observation, len(expected_observation))
    for player_id in range(len(expected_observation)):
      expected_player_observations = expected_observation[player_id]
      actual_player_observations = actual_observation[player_id]
      expected_keys = expected_player_observations.keys()
      actual_keys = actual_player_observations.keys()
      msg = ("Observation keys differ for player {}.\nExpected: {}.\nActual: {}"
             .format(player_id, expected_keys, actual_keys))
      self.assertEqual(expected_keys, actual_keys, msg)
      for key in expected_player_observations:
        expected_array = expected_player_observations[key]
        actual_array = actual_player_observations[key]
        msg = ("Observation {!r} differs for player {}.\nExpected:\n{}\n"
               "Actual:\n{}"
               .format(key, player_id, expected_array, actual_array))
        np.testing.assert_array_equal(expected_array, actual_array,
                                      err_msg=msg)

  @parameterized.parameters(True, False)
  def test_same_first_observation_if_same_seed(self, disable_walker_contacts):
    seed = 42
    timestep_1 = soccer.load(
        team_size=2,
        random_state=seed,
        disable_walker_contacts=disable_walker_contacts).reset()
    timestep_2 = soccer.load(
        team_size=2,
        random_state=seed,
        disable_walker_contacts=disable_walker_contacts).reset()
    self.assertSameObservation(timestep_1.observation, timestep_2.observation)

  @parameterized.parameters(True, False)
  def test_different_first_observation_if_different_seed(
      self, disable_walker_contacts):
    timestep_1 = soccer.load(
        team_size=2,
        random_state=1,
        disable_walker_contacts=disable_walker_contacts).reset()
    timestep_2 = soccer.load(
        team_size=2,
        random_state=2,
        disable_walker_contacts=disable_walker_contacts).reset()
    try:
      self.assertSameObservation(timestep_1.observation, timestep_2.observation)
    except AssertionError:
      pass
    else:
      self.fail("Observations are unexpectedly identical.")


if __name__ == "__main__":
  absltest.main()



# Copyright 2020 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tests for `dm_control.mjcf.export_with_assets`."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import zipfile

# Internal dependencies.

from absl import flags
from absl.testing import absltest
from absl.testing import parameterized
from dm_control import mjcf
from dm_control.mujoco.wrapper import util
import six

FLAGS = flags.FLAGS

_ASSETS_DIR = os.path.join(os.path.dirname(__file__), 'test_assets')
_TEST_MODEL_WITH_ASSETS = os.path.join(_ASSETS_DIR, 'model_with_assets.xml')
_TEST_MODEL_WITHOUT_ASSETS = os.path.join(_ASSETS_DIR, 'lego_brick.xml')


def setUpModule():
  # Flags are not parsed when this test is invoked by `nosetests`, so we fall
  # back on using the default value for `--test_tmpdir`.
  if not FLAGS.is_parsed():
    FLAGS.test_tmpdir = absltest.get_default_test_tmpdir()
    FLAGS.mark_as_parsed()


class ExportWithAssetsAsZipTest(parameterized.TestCase):

  @parameterized.named_parameters(
      ('with_assets', _TEST_MODEL_WITH_ASSETS, 'mujoco_with_assets'),
      ('without_assets', _TEST_MODEL_WITHOUT_ASSETS, 'mujoco'),
  )
  def test_export_model(self, xml_path, model_name):
    """Save processed MJCF model."""
    out_dir = self.create_tempdir().full_path
    mjcf_model = mjcf.from_path(xml_path)
    mjcf.export_with_assets_as_zip(
        mjcf_model, out_dir=out_dir, model_name=model_name)

    # Read the .zip file in the output directory.
    # Check that the only directory is named `model_name`/, and put all the
    # contents under any directory in a dict a directory in a dict.
    zip_file_contents = {}
    zip_filename = os.path.join(out_dir, (model_name + '.zip'))
    self.assertTrue(zipfile.is_zipfile(zip_filename))
    with zipfile.ZipFile(zip_filename, 'r') as zip_file:
      for zip_info in zip_file.infolist():
        # Note: zip_info.is_dir() is not Python 2 compatible, but directories
        # inside a ZipFile are guaranteed to end with '/'.
        if not zip_info.filename.endswith(os.path.sep):
          with zip_file.open(zip_info.filename) as f:
            zip_file_contents[zip_info.filename] = f.read()
        else:
          self.assertEqual(os.path.join(model_name), zip_info.filename)

    # Check that the output directory contains an XML file of the correct name.
    xml_filename = os.path.join(model_name, model_name) + '.xml'
    self.assertIn(xml_filename, zip_file_contents)

    # Check that its contents match the output of `mjcf_model.to_xml_string()`.
    xml_contents = util.to_native_string(zip_file_contents.pop(xml_filename))
    expected_xml_contents = mjcf_model.to_xml_string()
    self.assertEqual(xml_contents, expected_xml_contents)

    # Check that the other files in the directory match the contents of the
    # model's `assets` dict.
    assets = mjcf_model.get_assets()
    for asset_name, asset_contents in six.iteritems(assets):
      self.assertEqual(asset_contents,
                       zip_file_contents[os.path.join(model_name, asset_name)])

  def test_default_model_filename(self):
    out_dir = self.create_tempdir().full_path
    mjcf_model = mjcf.from_path(_TEST_MODEL_WITH_ASSETS)
    mjcf.export_with_assets_as_zip(mjcf_model, out_dir, model_name=None)
    expected_name = mjcf_model.model + '.zip'
    self.assertTrue(os.path.isfile(os.path.join(out_dir, expected_name)))


if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tests for the base windowing system."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from absl.testing import absltest
from dm_control.viewer import user_input
import mock


# pylint: disable=g-import-not-at-top
_OPEN_GL_MOCK = mock.MagicMock()
_MOCKED_MODULES = {
    'OpenGL': _OPEN_GL_MOCK,
    'OpenGL.GL': _OPEN_GL_MOCK,
}
with mock.patch.dict('sys.modules', _MOCKED_MODULES):
  from dm_control.viewer.gui import base
# pylint: enable=g-import-not-at-top

_EPSILON = 1e-7


@mock.patch(base.__name__ + '.time')
class DoubleClickDetectorTest(absltest.TestCase):

  def setUp(self):
    self.detector = base.DoubleClickDetector()
    self.double_click_event = (user_input.MOUSE_BUTTON_LEFT, user_input.PRESS)

  def test_two_rapid_clicks_yield_double_click_event(self, mock_time):
    mock_time.time.return_value = 0
    self.assertFalse(self.detector.process(*self.double_click_event))

    mock_time.time.return_value = base._DOUBLE_CLICK_INTERVAL - _EPSILON
    self.assertTrue(self.detector.process(*self.double_click_event))

  def test_two_slow_clicks_dont_yield_double_click_event(self, mock_time):
    mock_time.time.return_value = 0
    self.assertFalse(self.detector.process(*self.double_click_event))

    mock_time.time.return_value = base._DOUBLE_CLICK_INTERVAL
    self.assertFalse(self.detector.process(*self.double_click_event))

  def test_sequence_of_slow_clicks_followed_by_fast_click(self, mock_time):
    click_times = [(0., False),
                   (base._DOUBLE_CLICK_INTERVAL * 2., False),
                   (base._DOUBLE_CLICK_INTERVAL * 3. - _EPSILON, True)]
    for click_time, result in click_times:
      mock_time.time.return_value = click_time
      self.assertEqual(result, self.detector.process(*self.double_click_event))


if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tests of the viewer.py module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from absl.testing import absltest
from dm_control.mujoco.wrapper.mjbindings import enums
from dm_control.viewer import util
from dm_control.viewer import viewer
import mock


class ViewerTest(absltest.TestCase):

  def setUp(self):
    super(ViewerTest, self).setUp()
    self.viewport = mock.MagicMock()
    self.mouse = mock.MagicMock()
    self.keyboard = mock.MagicMock()
    self.viewer = viewer.Viewer(self.viewport, self.mouse, self.keyboard)

    self.viewer._render_settings = mock.MagicMock()
    self.physics = mock.MagicMock()
    self.renderer = mock.MagicMock()
    self.renderer.priority_components = util.QuietSet()

  def _extract_bind_call_args(self, bind_mock):
    call_args = []
    for calls in bind_mock.call_args_list:
      args = calls[0]
      if len(args) == 2:
        call_args.append(args[1])
    return call_args

  def test_initialize_creates_components(self):
    with mock.patch(viewer.__name__ + '.renderer'):
      self.viewer.initialize(self.physics, self.renderer, touchpad=False)
    self.assertIsNotNone(self.viewer._camera)
    self.assertIsNotNone(self.viewer._manipulator)
    self.assertIsNotNone(self.viewer._free_camera)
    self.assertIsNotNone(self.viewer._camera_select)
    self.assertEqual(self.renderer, self.viewer._renderer)

  def test_initialize_creates_touchpad_specific_input_mapping(self):
    self.viewer._input_map = mock.MagicMock()
    with mock.patch(viewer.__name__ + '.renderer'):
      self.viewer.initialize(self.physics, self.renderer, touchpad=True)
    call_args = self._extract_bind_call_args(self.viewer._input_map.bind)
    self.assertIn(viewer._MOVE_OBJECT_VERTICAL_TOUCHPAD, call_args)
    self.assertIn(viewer._MOVE_OBJECT_HORIZONTAL_TOUCHPAD, call_args)
    self.assertIn(viewer._ROTATE_OBJECT_TOUCHPAD, call_args)
    self.assertIn(viewer._PAN_CAMERA_VERTICAL_TOUCHPAD, call_args)
    self.assertIn(viewer._PAN_CAMERA_HORIZONTAL_TOUCHPAD, call_args)

  def test_initialize_create_mouse_specific_input_mapping(self):
    self.viewer._input_map = mock.MagicMock()
    with mock.patch(viewer.__name__ + '.renderer'):
      self.viewer.initialize(self.physics, self.renderer, touchpad=False)
    call_args = self._extract_bind_call_args(self.viewer._input_map.bind)
    self.assertIn(viewer._MOVE_OBJECT_VERTICAL_MOUSE, call_args)
    self.assertIn(viewer._MOVE_OBJECT_HORIZONTAL_MOUSE, call_args)
    self.assertIn(viewer._ROTATE_OBJECT_MOUSE, call_args)
    self.assertIn(viewer._PAN_CAMERA_VERTICAL_MOUSE, call_args)
    self.assertIn(viewer._PAN_CAMERA_HORIZONTAL_MOUSE, call_args)

  def test_initialization_flushes_old_input_map(self):
    self.viewer._input_map = mock.MagicMock()
    with mock.patch(viewer.__name__ + '.renderer'):
      self.viewer.initialize(self.physics, self.renderer, touchpad=False)
    self.viewer._input_map.clear_bindings.assert_called_once()

  def test_deinitialization_deletes_components(self):
    self.viewer._camera = mock.MagicMock()
    self.viewer._manipulator = mock.MagicMock()
    self.viewer._free_camera = mock.MagicMock()
    self.viewer._camera_select = mock.MagicMock()
    self.viewer._renderer = mock.MagicMock()
    self.viewer.deinitialize()
    self.assertIsNone(self.viewer._camera)
    self.assertIsNone(self.viewer._manipulator)
    self.assertIsNone(self.viewer._free_camera)
    self.assertIsNone(self.viewer._camera_select)
    self.assertIsNone(self.viewer._renderer)

  def test_deinitialization_flushes_old_input_map(self):
    self.viewer._input_map = mock.MagicMock()
    self.viewer.deinitialize()
    self.viewer._input_map.clear_bindings.assert_called_once()

  def test_rendering_uninitialized(self):
    self.viewer.render()  # nothing crashes

  def test_zoom_to_scene_uninitialized(self):
    self.viewer.zoom_to_scene()  # nothing crashes

  def test_rendering(self):
    self.viewer._camera = mock.MagicMock()
    self.viewer._renderer = mock.MagicMock()
    self.viewer.render()
    self.viewer._camera.render.assert_called_once_with(self.viewer.perturbation)
    self.viewer._renderer.render.assert_called_once()

  def test_applying_render_settings_before_rendering_a_scene(self):
    self.viewer._camera = mock.MagicMock()
    self.viewer._renderer = mock.MagicMock()
    self.viewer.render()
    self.viewer._render_settings.apply_settings.assert_called_once()

  def test_zoom_to_scene(self):
    self.viewer._camera = mock.MagicMock()
    self.viewer.zoom_to_scene()
    self.viewer._camera.zoom_to_scene.assert_called_once()

  def test_retrieving_perturbation(self):
    object_perturbation = mock.MagicMock()
    self.viewer._manipulator = mock.MagicMock()
    self.viewer._manipulator.perturbation = object_perturbation
    self.assertEqual(object_perturbation, self.viewer.perturbation)

  def test_retrieving_perturbation_without_manipulator(self):
    self.viewer._manipulator = None
    self.assertEqual(self.viewer._null_perturbation, self.viewer.perturbation)

  def test_retrieving_perturbation_without_selected_object(self):
    self.viewer._manipulator = mock.MagicMock()
    self.viewer._manipulator.perturbation = None
    self.assertEqual(self.viewer._null_perturbation, self.viewer.perturbation)


class CameraSelectorTest(absltest.TestCase):

  def setUp(self):
    super(CameraSelectorTest, self).setUp()
    self.camera = mock.MagicMock()
    self.model = mock.MagicMock()
    self.free_camera = mock.MagicMock()
    self.model.ncam = 2

    options = {
        'camera': self.camera,
        'model': self.model,
        'free_camera': self.free_camera
    }

    self.controller = viewer.CameraSelector(**options)

  def test_activating_freelook_camera_by_default(self):
    self.assertEqual(self.controller._free_ctrl, self.controller._active_ctrl)

  def test_cycling_forward_through_cameras(self):
    self.controller.select_next()
    self.assertIsNone(self.controller._active_ctrl)
    self.controller._free_ctrl.deactivate.assert_called_once()
    self.controller._free_ctrl.reset_mock()
    self.controller._camera.set_fixed_mode.assert_called_once_with(0)
    self.controller._camera.reset_mock()

    self.controller.select_next()
    self.assertIsNone(self.controller._active_ctrl)
    self.controller._camera.set_fixed_mode.assert_called_once_with(1)
    self.controller._camera.reset_mock()

    self.controller.select_next()
    self.assertEqual(self.controller._free_ctrl, self.controller._active_ctrl)
    self.controller._free_ctrl.activate.assert_called_once()

  def test_cycling_backwards_through_cameras(self):
    self.controller.select_previous()
    self.assertIsNone(self.controller._active_ctrl)
    self.controller._free_ctrl.deactivate.assert_called_once()
    self.controller._free_ctrl.reset_mock()
    self.controller._camera.set_fixed_mode.assert_called_once_with(1)
    self.controller._camera.reset_mock()

    self.controller.select_previous()
    self.assertIsNone(self.controller._active_ctrl)
    self.controller._camera.set_fixed_mode.assert_called_once_with(0)
    self.controller._camera.reset_mock()

    self.controller.select_previous()
    self.assertEqual(self.controller._free_ctrl, self.controller._active_ctrl)
    self.controller._free_ctrl.activate.assert_called_once()

  def test_controller_activation(self):
    old_controller = mock.MagicMock()
    new_controller = mock.MagicMock()
    self.controller._active_ctrl = old_controller
    self.controller._activate(new_controller)
    old_controller.deactivate.assert_called_once()
    new_controller.activate.assert_called_once()

  def test_controller_activation_not_repeated_for_already_active_one(self):
    controller = mock.MagicMock()
    self.controller._active_ctrl = controller
    self.controller._activate(controller)
    self.assertEqual(0, controller.deactivate.call_count)
    self.assertEqual(0, controller.activate.call_count)


class FreeCameraControllerTest(absltest.TestCase):

  def setUp(self):
    super(FreeCameraControllerTest, self).setUp()
    self.viewport = mock.MagicMock()
    self.camera = mock.MagicMock()
    self.mouse = mock.MagicMock()
    self.selection_service = mock.MagicMock()

    options = {
        'camera': self.camera,
        'viewport': self.viewport,
        'pointer': self.mouse,
        'selection_service': self.selection_service
    }

    self.controller = viewer.FreeCameraController(**options)
    self.controller._action = mock.MagicMock()

  def test_activation_while_not_in_tracking_mode(self):
    self.controller._tracked_body_idx = -1
    self.controller.activate()
    self.camera.set_freelook_mode.assert_called_once()

  def test_activation_while_in_tracking_mode(self):
    self.controller._tracked_body_idx = 1
    self.controller.activate()
    self.camera.set_tracking_mode.assert_called_once_with(1)

  def test_activation_and_deactivation_flag(self):
    self.controller.activate()
    self.assertTrue(self.controller._active)
    self.controller.deactivate()
    self.assertFalse(self.controller._active)

  def test_vertical_panning_camera_with_active_controller(self):
    self.controller._active = True
    self.controller.set_pan_vertical_mode(True)
    self.controller._action.begin.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_MOVE_V)
    self.controller.set_pan_vertical_mode(False)
    self.controller._action.end.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_MOVE_V)

  def test_vertical_panning_camera_with_inactive_controller(self):
    self.controller._active = False
    self.controller.set_pan_vertical_mode(True)
    self.assertEqual(0, self.controller._action.begin.call_count)
    self.controller.set_pan_vertical_mode(False)
    self.assertEqual(0, self.controller._action.end.call_count)

  def test_horizontal_panning_camera_with_active_controller(self):
    self.controller._active = True
    self.controller.set_pan_horizontal_mode(True)
    self.controller._action.begin.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_MOVE_H)
    self.controller.set_pan_horizontal_mode(False)
    self.controller._action.end.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_MOVE_H)

  def test_horizontal_panning_camera_with_inactive_controller(self):
    self.controller._active = False
    self.controller.set_pan_horizontal_mode(True)
    self.assertEqual(0, self.controller._action.begin.call_count)
    self.controller.set_pan_horizontal_mode(False)
    self.assertEqual(0, self.controller._action.end.call_count)

  def test_rotating_camera_with_active_controller(self):
    self.controller._active = True
    self.controller.set_rotate_mode(True)
    self.controller._action.begin.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_ROTATE_H)
    self.controller.set_rotate_mode(False)
    self.controller._action.end.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_ROTATE_H)

  def test_rotating_camera_with_inactive_controller(self):
    self.controller._active = False
    self.controller.set_rotate_mode(True)
    self.assertEqual(0, self.controller._action.begin.call_count)
    self.controller.set_rotate_mode(False)
    self.assertEqual(0, self.controller._action.end.call_count)

  def test_centering_with_active_controller(self):
    self.controller._active = True
    self.camera.raycast.return_value = 1, 2
    self.controller.center()
    self.camera.raycast.assert_called_once()

  def test_centering_with_inactive_controller(self):
    self.controller._active = False
    self.controller.center()
    self.assertEqual(0, self.camera.raycast.call_count)

  def test_moving_mouse_moves_camera(self):
    position = [100, 200]
    translation = [1, 0]
    viewport_space_translation = [2, 0]
    action = 1
    self.viewport.screen_to_viewport.return_value = viewport_space_translation
    self.controller._action.in_progress = True
    self.controller._action.watermark = action

    self.controller.on_move(position, translation)
    self.viewport.screen_to_viewport.assert_called_once_with(translation)
    self.camera.move.assert_called_once_with(action, viewport_space_translation)

  def test_mouse_move_doesnt_work_without_an_action_selected(self):
    self.controller._action.in_progress = False
    self.controller.on_move([], [])
    self.assertEqual(0, self.camera.move.call_count)

  def test_zoom_with_active_controller(self):
    self.controller._active = True
    expected_zoom_vector = [0, -0.05]
    self.controller.zoom(1.)
    self.camera.move.assert_called_once_with(
        enums.mjtMouse.mjMOUSE_ZOOM, expected_zoom_vector)

  def test_zoom_with_inactive_controller(self):
    self.controller._active = False
    self.controller.zoom(1.)
    self.assertEqual(0, self.camera.move.call_count)

  def test_tracking_with_active_controller(self):
    self.controller._active = True
    selected_body_id = 5
    self.selection_service.selected_body_id = selected_body_id
    self.controller._tracked_body_idx = -1

    self.controller.track()
    self.assertEqual(self.controller._tracked_body_idx, selected_body_id)
    self.camera.set_tracking_mode.assert_called_once_with(selected_body_id)

  def test_tracking_with_inactive_controller(self):
    self.controller._active = False
    selected_body_id = 5
    self.selection_service.selected_body_id = selected_body_id
    self.controller.track()
    self.assertEqual(self.controller._tracked_body_idx, -1)
    self.assertEqual(0, self.camera.set_tracking_mode.call_count)

  def test_free_look_mode_with_active_controller(self):
    self.controller._active = True
    self.controller._tracked_body_idx = 5
    self.controller.free_look()
    self.assertEqual(self.controller._tracked_body_idx, -1)
    self.camera.set_freelook_mode.assert_called_once()

  def test_free_look_mode_with_inactive_controller(self):
    self.controller._active = False
    self.controller._tracked_body_idx = 5
    self.controller.free_look()
    self.assertEqual(self.controller._tracked_body_idx, 5)
    self.assertEqual(0, self.camera.set_freelook_mode.call_count)


class ManipulationControllerTest(absltest.TestCase):

  def setUp(self):
    super(ManipulationControllerTest, self).setUp()
    self.viewport = mock.MagicMock()
    self.camera = mock.MagicMock()
    self.mouse = mock.MagicMock()

    options = {
        'camera': self.camera,
        'viewport': self.viewport,
        'pointer': self.mouse,
    }

    self.controller = viewer.ManipulationController(**options)

    self.body_id = 1
    self.click_pos_on_body = [1, 2, 3]
    self.camera.raycast.return_value = (self.body_id, self.click_pos_on_body)

  def test_selecting_a_body(self):
    self.camera.raycast.return_value = (self.body_id, self.click_pos_on_body)
    self.controller.select()
    self.assertIsNotNone(self.controller._perturb)

  def test_selecting_empty_space_cancels_selection(self):
    self.camera.raycast.return_value = (-1, None)
    self.controller.select()
    self.assertIsNone(self.controller._perturb)

  def test_vertical_movement_operation(self):
    self.controller._perturb = mock.MagicMock()

    self.controller.set_move_vertical_mode(True)
    self.controller._perturb.start_move.assert_called_once()
    self.assertEqual(enums.mjtMouse.mjMOUSE_MOVE_V,
                     self.controller._perturb.start_move.call_args[0][0])

    self.controller.set_move_vertical_mode(False)
    self.controller._perturb.end_move.assert_called_once()

  def test_horzontal_movement_operation(self):
    self.controller._perturb = mock.MagicMock()

    self.controller.set_move_horizontal_mode(True)
    self.controller._perturb.start_move.assert_called_once()
    self.assertEqual(enums.mjtMouse.mjMOUSE_MOVE_H,
                     self.controller._perturb.start_move.call_args[0][0])

    self.controller.set_move_horizontal_mode(False)
    self.controller._perturb.end_move.assert_called_once()

  def test_rotation_operation(self):
    self.controller._perturb = mock.MagicMock()

    self.controller.set_rotate_mode(True)
    self.controller._perturb.start_move.assert_called_once()
    self.assertEqual(enums.mjtMouse.mjMOUSE_ROTATE_H,
                     self.controller._perturb.start_move.call_args[0][0])

    self.controller.set_rotate_mode(False)
    self.controller._perturb.end_move.assert_called_once()

  def test_every_action_generates_a_fresh_grab_pos(self):
    some_action = 0
    self.controller._perturb = mock.MagicMock()
    self.controller._update_action(some_action)
    self.camera.raycast.assert_called_once()

  def test_actions_not_started_without_object_selected(self):
    some_action = 0
    self.controller._perturb = None
    self.controller._update_action(some_action)
    self.assertEqual(0, self.camera.raycast.call_count)

  def test_on_move_requires_an_action_to_be_started_first(self):
    self.controller._perturb = mock.MagicMock()
    self.controller._action = mock.MagicMock()
    self.controller._action.in_progress = False

    self.controller.on_move([], [])
    self.assertEqual(0, self.controller._perturb.tick_move.call_count)

  def test_dragging_selected_object_moves_it(self):
    screen_pos = [1, 2]
    screen_translation = [3, 4]
    viewport_offset = [5, 6]
    self.controller._perturb = mock.MagicMock()
    self.controller._action = mock.MagicMock()
    self.controller._action.in_progress = True
    self.viewport.screen_to_viewport.return_value = viewport_offset

    self.controller.on_move(screen_pos, screen_translation)
    self.viewport.screen_to_viewport.assert_called_once_with(screen_translation)
    self.controller._perturb.tick_move.assert_called_once_with(viewport_offset)

  def test_operations_require_object_to_be_selected(self):
    self.controller._perturb = None

    # No exceptions should be raised.
    self.controller.set_move_vertical_mode(True)
    self.controller.set_move_vertical_mode(False)
    self.controller.set_move_horizontal_mode(True)
    self.controller.set_move_horizontal_mode(False)
    self.controller.set_rotate_mode(True)
    self.controller.set_rotate_mode(False)
    self.controller.on_move([1, 2], [3, 4])


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Arenas for Locomotion tasks."""

from dm_control.locomotion.arenas.bowl import Bowl
from dm_control.locomotion.arenas.corridors import EmptyCorridor
from dm_control.locomotion.arenas.corridors import GapsCorridor
from dm_control.locomotion.arenas.corridors import WallsCorridor
from dm_control.locomotion.arenas.floors import Floor
from dm_control.locomotion.arenas.labmaze_textures import FloorTextures
from dm_control.locomotion.arenas.labmaze_textures import SkyBox
from dm_control.locomotion.arenas.labmaze_textures import WallTextures
from dm_control.locomotion.arenas.mazes import MazeWithTargets
from dm_control.locomotion.arenas.mazes import RandomMazeWithTargets



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Functions for parsing XML into an MJCF object model."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys

from dm_control.mjcf import constants
from dm_control.mjcf import debugging
from dm_control.mjcf import element
from lxml import etree
import six
from dm_control.utils import io as resources


def from_xml_string(xml_string, escape_separators=False,
                    model_dir='', resolve_references=True, assets=None):
  """Parses an XML string into an MJCF object model.

  Args:
    xml_string: An XML string representing an MJCF model.
    escape_separators: (optional) A boolean, whether to replace '/' characters
      in element identifiers. If `False`, any '/' present in the XML causes
      a ValueError to be raised.
    model_dir: (optional) Path to the directory containing the model XML file.
      This is used to prefix the paths of all asset files.
    resolve_references: (optional) A boolean indicating whether the parser
      should attempt to resolve reference attributes to a corresponding element.
    assets: (optional) A dictionary of pre-loaded assets, of the form
      `{filename: bytestring}`. If present, PyMJCF will search for assets in
      this dictionary before attempting to load them from the filesystem.

  Returns:
    An `mjcf.RootElement`.
  """
  xml_root = etree.fromstring(xml_string)
  return _parse(xml_root, escape_separators,
                model_dir=model_dir,
                resolve_references=resolve_references,
                assets=assets)


def from_file(file_handle, escape_separators=False,
              model_dir='', resolve_references=True, assets=None):
  """Parses an XML file into an MJCF object model.

  Args:
    file_handle: A Python file-like handle.
    escape_separators: (optional) A boolean, whether to replace '/' characters
      in element identifiers. If `False`, any '/' present in the XML causes
      a ValueError to be raised.
    model_dir: (optional) Path to the directory containing the model XML file.
      This is used to prefix the paths of all asset files.
    resolve_references: (optional) A boolean indicating whether the parser
      should attempt to resolve reference attributes to a corresponding element.
    assets: (optional) A dictionary of pre-loaded assets, of the form
      `{filename: bytestring}`. If present, PyMJCF will search for assets in
      this dictionary before attempting to load them from the filesystem.

  Returns:
    An `mjcf.RootElement`.
  """
  xml_root = etree.parse(file_handle).getroot()
  return _parse(xml_root, escape_separators,
                model_dir=model_dir,
                resolve_references=resolve_references,
                assets=assets)


def from_path(path, escape_separators=False, resolve_references=True,
              assets=None):
  """Parses an XML file into an MJCF object model.

  Args:
    path: A path to an XML file. This path should be loadable using
      `resources.GetResource`.
    escape_separators: (optional) A boolean, whether to replace '/' characters
      in element identifiers. If `False`, any '/' present in the XML causes
      a ValueError to be raised.
    resolve_references: (optional) A boolean indicating whether the parser
      should attempt to resolve reference attributes to a corresponding element.
    assets: (optional) A dictionary of pre-loaded assets, of the form
      `{filename: bytestring}`. If present, PyMJCF will search for assets in
      this dictionary before attempting to load them from the filesystem.

  Returns:
    An `mjcf.RootElement`.
  """
  model_dir, _ = os.path.split(path)
  contents = resources.GetResource(path)
  xml_root = etree.fromstring(contents)
  return _parse(xml_root, escape_separators,
                model_dir=model_dir, resolve_references=resolve_references,
                assets=assets)


def _parse(xml_root, escape_separators=False,
           model_dir='', resolve_references=True, assets=None):
  """Parses a complete MJCF model from an XML.

  Args:
    xml_root: An `etree.Element` object.
    escape_separators: (optional) A boolean, whether to replace '/' characters
      in element identifiers. If `False`, any '/' present in the XML causes
      a ValueError to be raised.
    model_dir: (optional) Path to the directory containing the model XML file.
      This is used to prefix the paths of all asset files.
    resolve_references: (optional) A boolean indicating whether the parser
      should attempt to resolve reference attributes to a corresponding element.
    assets: (optional) A dictionary of pre-loaded assets, of the form
      `{filename: bytestring}`. If present, PyMJCF will search for assets in
      this dictionary before attempting to load them from the filesystem.

  Returns:
    An `mjcf.RootElement`.

  Raises:
    ValueError: If `xml_root`'s tag is not 'mujoco'.
  """

  assets = assets or {}

  if xml_root.tag != 'mujoco':
    raise ValueError('Root element of the XML should be <mujoco>: got <{}>'
                     .format(xml_root.tag))

  with debugging.freeze_current_stack_trace():
    # Recursively parse any included XML files.
    to_include = []
    for include_tag in xml_root.findall('include'):
      try:
        # First look for the path to the included XML file in the assets dict.
        path_or_xml_string = assets[include_tag.attrib['file']]
        parsing_func = from_xml_string
      except KeyError:
        # If it's not present in the assets dict then attempt to load the XML
        # from the filesystem.
        path_or_xml_string = os.path.join(model_dir, include_tag.attrib['file'])
        parsing_func = from_path
      included_mjcf = parsing_func(
          path_or_xml_string,
          escape_separators=escape_separators,
          resolve_references=resolve_references,
          assets=assets)
      to_include.append(included_mjcf)
      # We must remove <include/> tags before parsing the main XML file, since
      # these are a schema violation.
      xml_root.remove(include_tag)

    # Parse the main XML file.
    try:
      model = xml_root.attrib.pop('model')
    except KeyError:
      model = None
    mjcf_root = element.RootElement(
        model=model, model_dir=model_dir, assets=assets)
    _parse_children(xml_root, mjcf_root, escape_separators)

    # Merge in the included XML files.
    for included_mjcf in to_include:
      # The included MJCF might have been automatically assigned a model name
      # that conficts with that of `mjcf_root`, so we override it here.
      included_mjcf.model = mjcf_root.model
      mjcf_root.include_copy(included_mjcf)

    if resolve_references:
      mjcf_root.resolve_references()
    return mjcf_root


def _parse_children(xml_element, mjcf_element, escape_separators=False):
  """Parses all children of a given XML element into an MJCF element.

  Args:
    xml_element: The source `etree.Element` object.
    mjcf_element: The target `mjcf.Element` object.
    escape_separators: (optional) A boolean, whether to replace '/' characters
      in element identifiers. If `False`, any '/' present in the XML causes
      a ValueError to be raised.
  """
  for xml_child in xml_element:
    if xml_child.tag is etree.Comment or xml_child.tag is etree.PI:
      continue
    try:
      child_spec = mjcf_element.spec.children[xml_child.tag]
      if escape_separators:
        attributes = {}
        for name, value in six.iteritems(xml_child.attrib):
          new_value = value.replace(
              constants.PREFIX_SEPARATOR_ESCAPE,
              constants.PREFIX_SEPARATOR_ESCAPE * 2)
          new_value = new_value.replace(
              constants.PREFIX_SEPARATOR, constants.PREFIX_SEPARATOR_ESCAPE)
          attributes[name] = new_value
      else:
        attributes = dict(xml_child.attrib)
      if child_spec.repeated or child_spec.on_demand:
        mjcf_child = mjcf_element.add(xml_child.tag, **attributes)
      else:
        mjcf_child = getattr(mjcf_element, xml_child.tag)
        mjcf_child.set_attributes(**attributes)
    except:  # pylint: disable=bare-except
      err_type, err, traceback = sys.exc_info()
      message = ('Line {}: error while parsing element <{}>: {}'
                 .format(xml_child.sourceline, xml_child.tag, err))
      six.reraise(err_type, err_type(message), traceback)
    _parse_children(xml_child, mjcf_child, escape_separators)



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Acrobot domain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.utils import containers
from dm_control.utils import rewards
import numpy as np

_DEFAULT_TIME_LIMIT = 10
SUITE = containers.TaggedTasks()


def get_model_and_assets():
  """Returns a tuple containing the model XML string and a dict of assets."""
  return common.read_model('acrobot.xml'), common.ASSETS


@SUITE.add('benchmarking')
def swingup(time_limit=_DEFAULT_TIME_LIMIT, random=None,
            environment_kwargs=None):
  """Returns Acrobot balance task."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = Balance(sparse=False, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


@SUITE.add('benchmarking')
def swingup_sparse(time_limit=_DEFAULT_TIME_LIMIT, random=None,
                   environment_kwargs=None):
  """Returns Acrobot sparse balance."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = Balance(sparse=True, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


class Physics(mujoco.Physics):
  """Physics simulation with additional features for the Acrobot domain."""

  def horizontal(self):
    """Returns horizontal (x) component of body frame z-axes."""
    return self.named.data.xmat[['upper_arm', 'lower_arm'], 'xz']

  def vertical(self):
    """Returns vertical (z) component of body frame z-axes."""
    return self.named.data.xmat[['upper_arm', 'lower_arm'], 'zz']

  def to_target(self):
    """Returns the distance from the tip to the target."""
    tip_to_target = (self.named.data.site_xpos['target'] -
                     self.named.data.site_xpos['tip'])
    return np.linalg.norm(tip_to_target)

  def orientations(self):
    """Returns the sines and cosines of the pole angles."""
    return np.concatenate((self.horizontal(), self.vertical()))


class Balance(base.Task):
  """An Acrobot `Task` to swing up and balance the pole."""

  def __init__(self, sparse, random=None):
    """Initializes an instance of `Balance`.

    Args:
      sparse: A `bool` specifying whether to use a sparse (indicator) reward.
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).
    """
    self._sparse = sparse
    super(Balance, self).__init__(random=random)

  def initialize_episode(self, physics):
    """Sets the state of the environment at the start of each episode.

    Shoulder and elbow are set to a random position between [-pi, pi).

    Args:
      physics: An instance of `Physics`.
    """
    physics.named.data.qpos[
        ['shoulder', 'elbow']] = self.random.uniform(-np.pi, np.pi, 2)
    super(Balance, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns an observation of pole orientation and angular velocities."""
    obs = collections.OrderedDict()
    obs['orientations'] = physics.orientations()
    obs['velocity'] = physics.velocity()
    return obs

  def _get_reward(self, physics, sparse):
    target_radius = physics.named.model.site_size['target', 0]
    return rewards.tolerance(physics.to_target(),
                             bounds=(0, target_radius),
                             margin=0 if sparse else 1)

  def get_reward(self, physics):
    """Returns a sparse or a smooth reward, as specified in the constructor."""
    return self._get_reward(physics, sparse=self._sparse)



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Components and views that render custom images into Mujoco render frame."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc

from dm_control.mujoco.wrapper import mjbindings
from dm_control.mujoco.wrapper import util
from dm_control.viewer import renderer
import enum
import numpy as np
import six

enums = mjbindings.enums
mjlib = mjbindings.mjlib
types = mjbindings.types


class PanelLocation(enum.Enum):
  TOP_LEFT = enums.mjtGridPos.mjGRID_TOPLEFT
  TOP_RIGHT = enums.mjtGridPos.mjGRID_TOPRIGHT
  BOTTOM_LEFT = enums.mjtGridPos.mjGRID_BOTTOMLEFT
  BOTTOM_RIGHT = enums.mjtGridPos.mjGRID_BOTTOMRIGHT


@six.add_metaclass(abc.ABCMeta)
class BaseViewportView(object):
  """Base abstract view class."""

  @abc.abstractmethod
  def render(self, context, viewport, location):
    """Renders the view on screen.

    Args:
      context: MjrContext instance.
      viewport: Viewport instance.
      location: Value defined in PanelLocation enum.
    """
    pass


@six.add_metaclass(abc.ABCMeta)
class ColumnTextModel(object):
  """Data model that returns 2 columns of text."""

  @abc.abstractmethod
  def get_columns(self):
    """Returns the text to display in two columns.

    Returns:
      Returns an iterable of tuples of 2 strings. Each tuple has format
      (left_column_label, right_column_label).
    """
    pass


class ColumnTextView(BaseViewportView):
  """A view displayed in Mujoco render window."""

  def __init__(self, model):
    """Instance initializer.

    Args:
      model: Instance of ColumnTextModel.
    """
    self._model = model

  def render(self, context, viewport, location):
    """Renders the overlay on screen.

    Args:
      context: MjrContext instance.
      viewport: Viewport instance.
      location: Value defined in PanelLocation enum.
    """
    columns = self._model.get_columns()
    if not columns:
      return

    columns = np.asarray(columns)
    left_column = '\n'.join(columns[:, 0])
    right_column = '\n'.join(columns[:, 1])
    mjlib.mjr_overlay(
        enums.mjtFont.mjFONT_NORMAL, location.value,
        viewport.mujoco_rect, util.to_binary_string(left_column),
        util.to_binary_string(right_column), context.ptr)


class MujocoDepthBuffer(renderer.Component):
  """Displays the contents of the scene's depth buffer."""

  def __init__(self):
    self._depth_buffer = np.empty((1, 1), np.float32)

  def render(self, context, viewport):
    """Renders the overlay on screen.

    Args:
      context: MjrContext instance.
      viewport: MJRRECT instance.
    """
    width_adjustment = viewport.width % 4
    rect_shape = (viewport.width - width_adjustment, viewport.height)

    if self._depth_buffer is None or self._depth_buffer.shape != rect_shape:
      self._depth_buffer = np.empty(
          (viewport.width, viewport.height), np.float32)

    mjlib.mjr_readPixels(
        None, self._depth_buffer, viewport.mujoco_rect, context.ptr)

    # Subsample by 4, convert to RGB, and cast to unsigned bytes.
    depth_rgb = np.repeat(self._depth_buffer[::4, ::4, None] * 255, 3,
                          -1).astype(np.ubyte)

    pos = types.MJRRECT(
        int(3 * viewport.width / 4) + width_adjustment, 0,
        int(viewport.width / 4), int(viewport.height / 4))
    mjlib.mjr_drawPixels(depth_rgb, None, pos, context.ptr)


class ViewportLayout(renderer.Component):
  """Layout manager for the render viewport.

  Allows to create a viewport layout by injecting renderer component even in
  absence of a renderer, and then easily reattach it between renderers.
  """

  def __init__(self):
    """Instance initializer."""
    self._views = dict()

  def __len__(self):
    return len(self._views)

  def __contains__(self, key):
    value = self._views.get(key, None)
    return value is not None

  def add(self, view, location):
    """Adds a new view.

    Args:
      view: renderer.BaseViewportView instance.
      location: Value defined in PanelLocation enum, location of the view in the
        viewport.
    """
    if not isinstance(view, BaseViewportView):
      raise TypeError(
          'View added to this layout needs to implement BaseViewportView.')
    self._views[view] = location

  def remove(self, view):
    """Removes a view.

    Args:
      view: renderer.BaseViewportView instance.
    """
    self._views.pop(view, None)

  def clear(self):
    """Removes all attached components."""
    self._views = dict()

  def render(self, context, viewport):
    """Renders the overlay on screen.

    Args:
      context: MjrContext instance.
      viewport: MJRRECT instance.
    """
    for view, location in six.iteritems(self._views):
      view.render(context, viewport, location)



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Simple floor arenas."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control import composer
from dm_control.locomotion.arenas import assets as locomotion_arenas_assets
import numpy as np

_TOP_CAMERA_DISTANCE = 100
_TOP_CAMERA_Y_PADDING_FACTOR = 1.1


class Floor(composer.Arena):
  """A simple floor arena with a checkered pattern."""

  def _build(self, size=(8, 8), reflectance=.2, aesthetic='default',
             name='floor'):
    super(Floor, self)._build(name=name)
    self._size = size

    self._mjcf_root.visual.headlight.set_attributes(
        ambient=[.4, .4, .4], diffuse=[.8, .8, .8], specular=[.1, .1, .1])

    if aesthetic != 'default':
      ground_info = locomotion_arenas_assets.get_ground_texture_info(aesthetic)
      sky_info = locomotion_arenas_assets.get_sky_texture_info(aesthetic)
      texturedir = locomotion_arenas_assets.get_texturedir(aesthetic)
      self._mjcf_root.compiler.texturedir = texturedir

      self._ground_texture = self._mjcf_root.asset.add(
          'texture', name='aesthetic_texture', file=ground_info.file,
          type=ground_info.type)
      self._ground_material = self._mjcf_root.asset.add(
          'material', name='aesthetic_material', texture=self._ground_texture,
          texuniform='true')
      self._skybox = self._mjcf_root.asset.add(
          'texture', name='aesthetic_skybox', file=sky_info.file,
          type='skybox', gridsize=sky_info.gridsize,
          gridlayout=sky_info.gridlayout)
    else:
      self._ground_texture = self._mjcf_root.asset.add(
          'texture',
          rgb1=[.2, .3, .4],
          rgb2=[.1, .2, .3],
          type='2d',
          builtin='checker',
          name='groundplane',
          width=300,
          height=300,
          mark='edge',
          markrgb=[0.8, 0.8, 0.8])
      self._ground_material = self._mjcf_root.asset.add(
          'material',
          name='groundplane',
          texrepeat=[3, 3],
          texuniform=True,
          reflectance=reflectance,
          texture=self._ground_texture)

    # Build groundplane.
    self._ground_geom = self._mjcf_root.worldbody.add(
        'geom',
        type='plane',
        name='groundplane',
        material=self._ground_material,
        size=list(size) + [0.5])

    # Choose the FOV so that the floor always fits nicely within the frame
    # irrespective of actual floor size.
    fovy_radians = 2 * np.arctan2(_TOP_CAMERA_Y_PADDING_FACTOR * size[1],
                                  _TOP_CAMERA_DISTANCE)
    self._top_camera = self._mjcf_root.worldbody.add(
        'camera',
        name='top_camera',
        pos=[0, 0, _TOP_CAMERA_DISTANCE],
        zaxis=[0, 0, 1],
        fovy=np.rad2deg(fovy_radians))

  @property
  def ground_geoms(self):
    return (self._ground_geom,)

  def regenerate(self, random_state):
    pass

  @property
  def size(self):
    return self._size



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for `engine`."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import unittest
# Internal dependencies.
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.mujoco import engine
from dm_control.mujoco import wrapper
from dm_control.mujoco.testing import assets
from dm_control.mujoco.wrapper import mjbindings
from dm_control.mujoco.wrapper.mjbindings import enums
from dm_control.rl import control
import mock
import numpy as np
import six
from six.moves import cPickle
from six.moves import range

mjlib = mjbindings.mjlib

MODEL_PATH = assets.get_path('cartpole.xml')
MODEL_WITH_ASSETS = assets.get_contents('model_with_assets.xml')
ASSETS = {
    'texture.png': assets.get_contents('deepmind.png'),
    'mesh.stl': assets.get_contents('cube.stl'),
    'included.xml': assets.get_contents('sphere.xml')
}


class MujocoEngineTest(parameterized.TestCase):

  def setUp(self):
    super(MujocoEngineTest, self).setUp()
    self._physics = engine.Physics.from_xml_path(MODEL_PATH)

  def _assert_attributes_equal(self, actual_obj, expected_obj, attr_to_compare):
    for name in attr_to_compare:
      actual_value = getattr(actual_obj, name)
      expected_value = getattr(expected_obj, name)
      try:
        if isinstance(expected_value, np.ndarray):
          np.testing.assert_array_equal(actual_value, expected_value)
        else:
          self.assertEqual(actual_value, expected_value)
      except AssertionError as e:
        raise AssertionError("Attribute '{}' differs from expected value. {}"
                             "".format(name, e.message))

  @parameterized.parameters(0, 'cart', u'cart')
  def testCameraIndexing(self, camera_id):
    height, width = 480, 640
    _ = engine.Camera(
        self._physics, height, width, camera_id=camera_id)

  def testDepthRender(self):
    plane_and_box = """
    <mujoco>
      <worldbody>
        <geom type="plane" pos="0 0 0" size="2 2 .1"/>
        <geom type="box" size=".1 .1 .1" pos="0 0 .1"/>
        <camera name="top" pos="0 0 3"/>
      </worldbody>
    </mujoco>
    """
    physics = engine.Physics.from_xml_string(plane_and_box)
    pixels = physics.render(height=200, width=200, camera_id='top', depth=True)
    # Nearest pixels should be 2.8m away
    np.testing.assert_approx_equal(pixels.min(), 2.8, 3)
    # Furthest pixels should be 3m away (depth is orthographic)
    np.testing.assert_approx_equal(pixels.max(), 3.0, 3)

  @parameterized.parameters([True, False])
  def testSegmentationRender(self, enable_geom_frame_rendering):
    box_four_corners = """
    <mujoco>
      <visual>
        <scale framelength="2"/>
      </visual>
      <worldbody>
        <geom name="box0" type="box" size=".2 .2 .2" pos="-1 1 .1"/>
        <geom name="box1" type="box" size=".2 .2 .2" pos="1 1 .1"/>
        <site name="box2" type="box" size=".2 .2 .2" pos="1 -1 .1"/>
        <site name="box3" type="box" size=".2 .2 .2" pos="-1 -1 .1"/>
        <camera name="top" pos="0 0 3"/>
      </worldbody>
    </mujoco>
    """
    physics = engine.Physics.from_xml_string(box_four_corners)
    obj_type_geom = enums.mjtObj.mjOBJ_GEOM  # Geom object type
    obj_type_site = enums.mjtObj.mjOBJ_SITE  # Site object type
    obj_type_decor = enums.mjtObj.mjOBJ_UNKNOWN  # Decor object type
    scene_options = wrapper.MjvOption()
    if enable_geom_frame_rendering:
      scene_options.frame = wrapper.mjbindings.enums.mjtFrame.mjFRAME_GEOM
    pixels = physics.render(height=200, width=200, camera_id='top',
                            segmentation=True, scene_option=scene_options)

    # The pixel indices below were chosen so that toggling the frame decors do
    # not affect the segmentation results.
    with self.subTest('Center pixels should have background label'):
      np.testing.assert_equal(pixels[95:105, 95:105, 0], -1)
      np.testing.assert_equal(pixels[95:105, 95:105, 1], -1)
    with self.subTest('Geoms have correct object type'):
      np.testing.assert_equal(pixels[15:25, 0:10, 1], obj_type_geom)
      np.testing.assert_equal(pixels[15:25, 190:200, 1], obj_type_geom)
    with self.subTest('Sites have correct object type'):
      np.testing.assert_equal(pixels[190:200, 190:200, 1], obj_type_site)
      np.testing.assert_equal(pixels[190:200, 0:10, 1], obj_type_site)
    with self.subTest('Geoms have correct object IDs'):
      np.testing.assert_equal(pixels[15:25, 0:10, 0],
                              physics.model.name2id('box0', obj_type_geom))
      np.testing.assert_equal(pixels[15:25, 190:200, 0],
                              physics.model.name2id('box1', obj_type_geom))
    with self.subTest('Sites have correct object IDs'):
      np.testing.assert_equal(pixels[190:200, 190:200, 0],
                              physics.model.name2id('box2', obj_type_site))
      np.testing.assert_equal(pixels[190:200, 0:10, 0],
                              physics.model.name2id('box3', obj_type_site))
    with self.subTest('Decor elements present if and only if geom frames are '
                      'enabled'):
      contains_decor = np.any(pixels[:, :, 1] == obj_type_decor)
      self.assertEqual(contains_decor, enable_geom_frame_rendering)

  def testTextOverlay(self):
    height, width = 480, 640
    overlay = engine.TextOverlay(title='Title', body='Body', style='big',
                                 position='bottom right')

    no_overlay = self._physics.render(height, width, camera_id=0)
    with_overlay = self._physics.render(height, width, camera_id=0,
                                        overlays=[overlay])
    self.assertFalse(np.all(no_overlay == with_overlay),
                     msg='Images are identical with and without text overlay.')

  def testSceneOption(self):
    height, width = 480, 640
    scene_option = wrapper.MjvOption()
    mjlib.mjv_defaultOption(scene_option.ptr)

    # Render geoms as semi-transparent.
    scene_option.flags[enums.mjtVisFlag.mjVIS_TRANSPARENT] = 1

    no_scene_option = self._physics.render(height, width, camera_id=0)
    with_scene_option = self._physics.render(height, width, camera_id=0,
                                             scene_option=scene_option)
    self.assertFalse(np.all(no_scene_option == with_scene_option),
                     msg='Images are identical with and without scene option.')

  def testRenderFlags(self):
    height, width = 480, 640
    cam = engine.Camera(self._physics, height, width, camera_id=0)
    cam.scene.flags[enums.mjtRndFlag.mjRND_WIREFRAME] = 1  # Enable wireframe
    enabled = cam.render().copy()
    cam.scene.flags[enums.mjtRndFlag.mjRND_WIREFRAME] = 0  # Disable wireframe
    disabled = cam.render().copy()
    self.assertFalse(
        np.all(disabled == enabled),
        msg='Images are identical regardless of whether wireframe is enabled.')

  @parameterized.parameters(((0.5, 0.5), (1, 3)),  # pole
                            ((0.5, 0.1), (0, 0)),  # ground
                            ((0.9, 0.9), (None, None)),  # sky
                           )
  def testCameraSelection(self, coordinates, expected_selection):
    height, width = 480, 640
    camera = engine.Camera(self._physics, height, width, camera_id=0)

    # Test for b/63380170: Enabling visualization of body frames adds
    # "non-model" geoms to the scene. This means that the indices of geoms
    # within `camera._scene.geoms` don't match the rows of `model.geom_bodyid`.
    camera.option.frame = enums.mjtFrame.mjFRAME_BODY

    selected = camera.select(coordinates)
    self.assertEqual(expected_selection, selected[:2])

  def testMovableCameraSetGetPose(self):
    height, width = 240, 320

    camera = engine.MovableCamera(self._physics, height, width)
    image = camera.render().copy()

    pose = camera.get_pose()

    lookat_offset = np.array([0.01, 0.02, -0.03])

    # Would normally pass the new values directly to camera.set_pose instead of
    # using the namedtuple _replace method, but this makes the asserts at the
    # end of the test a little cleaner.
    new_pose = pose._replace(distance=pose.distance * 1.5,
                             lookat=pose.lookat + lookat_offset,
                             azimuth=pose.azimuth + -15,
                             elevation=pose.elevation - 10)

    camera.set_pose(*new_pose)

    self.assertEqual(new_pose.distance, camera.get_pose().distance)
    self.assertEqual(new_pose.azimuth, camera.get_pose().azimuth)
    self.assertEqual(new_pose.elevation, camera.get_pose().elevation)
    np.testing.assert_allclose(new_pose.lookat, camera.get_pose().lookat)

    self.assertFalse(np.all(image == camera.render()))

  def testRenderExceptions(self):
    max_width = self._physics.model.vis.global_.offwidth
    max_height = self._physics.model.vis.global_.offheight
    max_camid = self._physics.model.ncam - 1
    with six.assertRaisesRegex(self, ValueError, 'width'):
      self._physics.render(max_height, max_width + 1, camera_id=max_camid)
    with six.assertRaisesRegex(self, ValueError, 'height'):
      self._physics.render(max_height + 1, max_width, camera_id=max_camid)
    with six.assertRaisesRegex(self, ValueError, 'camera_id'):
      self._physics.render(max_height, max_width, camera_id=max_camid + 1)
    with six.assertRaisesRegex(self, ValueError, 'camera_id'):
      self._physics.render(max_height, max_width, camera_id=-2)

  def testPhysicsRenderMethod(self):
    height, width = 240, 320
    image = self._physics.render(height=height, width=width)
    self.assertEqual(image.shape, (height, width, 3))
    depth = self._physics.render(height=height, width=width, depth=True)
    self.assertEqual(depth.shape, (height, width))
    segmentation = self._physics.render(height=height, width=width,
                                        segmentation=True)
    self.assertEqual(segmentation.shape, (height, width, 2))

  def testExceptionIfBothDepthAndSegmentation(self):
    with self.assertRaisesWithLiteralMatch(
        ValueError, engine._BOTH_SEGMENTATION_AND_DEPTH_ENABLED):
      self._physics.render(depth=True, segmentation=True)

  def testRenderFlagOverridesAreNotPersistent(self):
    camera = engine.Camera(self._physics)
    first_rgb = camera.render().copy()
    camera.render(segmentation=True)
    second_rgb = camera.render().copy()
    np.testing.assert_array_equal(first_rgb, second_rgb)

  def testCustomRenderFlags(self):
    default = self._physics.render()
    wireframe_string_key = self._physics.render(
        render_flag_overrides=dict(wireframe=True))
    self.assertFalse((default == wireframe_string_key).all())
    wireframe_enum_key = self._physics.render(
        render_flag_overrides={enums.mjtRndFlag.mjRND_WIREFRAME: True})
    np.testing.assert_array_equal(wireframe_string_key, wireframe_enum_key)

  @parameterized.parameters(dict(depth=True), dict(segmentation=True))
  def testExceptionIfRenderFlagOverridesAndDepthOrSegmentation(self, **kwargs):
    with self.assertRaisesWithLiteralMatch(
        ValueError,
        engine._RENDER_FLAG_OVERRIDES_NOT_SUPPORTED_FOR_DEPTH_OR_SEGMENTATION):
      self._physics.render(render_flag_overrides=dict(wireframe=True), **kwargs)

  def testExceptionIfOverlaysAndDepthOrSegmentation(self):
    overlay = engine.TextOverlay()
    with self.assertRaisesWithLiteralMatch(
        ValueError, engine._OVERLAYS_NOT_SUPPORTED_FOR_DEPTH_OR_SEGMENTATION):
      self._physics.render(depth=True, overlays=[overlay])
    with self.assertRaisesWithLiteralMatch(
        ValueError, engine._OVERLAYS_NOT_SUPPORTED_FOR_DEPTH_OR_SEGMENTATION):
      self._physics.render(segmentation=True, overlays=[overlay])

  def testNamedViews(self):
    self.assertEqual((1,), self._physics.control().shape)
    self.assertEqual((2,), self._physics.position().shape)
    self.assertEqual((2,), self._physics.velocity().shape)
    self.assertEqual((0,), self._physics.activation().shape)
    self.assertEqual((4,), self._physics.state().shape)
    self.assertEqual(0., self._physics.time())
    self.assertEqual(0.01, self._physics.timestep())

  def testSetGetPhysicsState(self):
    physics_state = self._physics.get_state()
    self._physics.set_state(physics_state)

    new_physics_state = np.random.random_sample(physics_state.shape)
    self._physics.set_state(new_physics_state)

    np.testing.assert_allclose(new_physics_state,
                               self._physics.get_state())

  def testSetInvalidPhysicsState(self):
    badly_shaped_state = np.repeat(self._physics.get_state(), repeats=2)

    with self.assertRaises(ValueError):
      self._physics.set_state(badly_shaped_state)

  def testNamedIndexing(self):
    self.assertEqual((3,), self._physics.named.data.xpos['cart'].shape)
    self.assertEqual((2, 3),
                     self._physics.named.data.xpos[['cart', 'pole']].shape)

  def testReload(self):
    self._physics.reload_from_xml_path(MODEL_PATH)

  def testReset(self):
    self._physics.reset()
    self.assertEqual(self._physics.data.qpos[1], 0)
    keyframe_id = 0
    self._physics.reset(keyframe_id=keyframe_id)
    self.assertEqual(self._physics.data.qpos[1],
                     self._physics.model.key_qpos[keyframe_id, 1])
    out_of_range = [-1, 3]
    max_valid = self._physics.model.nkey - 1
    for actual in out_of_range:
      with self.assertRaisesWithLiteralMatch(
          ValueError,
          engine._KEYFRAME_ID_OUT_OF_RANGE.format(
              max_valid=max_valid, actual=actual)):
        self._physics.reset(keyframe_id=actual)

  def testLoadAndReloadFromStringWithAssets(self):
    physics = engine.Physics.from_xml_string(
        MODEL_WITH_ASSETS, assets=ASSETS)
    physics.reload_from_xml_string(MODEL_WITH_ASSETS, assets=ASSETS)

  def testFree(self):
    def mock_free(obj):
      return mock.patch.object(obj, 'free', wraps=obj.free)

    with mock_free(self._physics.model) as mock_free_model:
      with mock_free(self._physics.data) as mock_free_data:
        with mock_free(self._physics.contexts.mujoco) as mock_free_mjrcontext:
          self._physics.free()

    mock_free_model.assert_called_once()
    mock_free_data.assert_called_once()
    mock_free_mjrcontext.assert_called_once()
    self.assertIsNone(self._physics.model.ptr)
    self.assertIsNone(self._physics.data.ptr)

  @parameterized.parameters(*enums.mjtWarning._fields[:-1])
  def testDivergenceException(self, warning_name):
    warning_enum = getattr(enums.mjtWarning, warning_name)
    with self.assertRaisesWithLiteralMatch(
        control.PhysicsError,
        engine._INVALID_PHYSICS_STATE.format(warning_names=warning_name)):
      with self._physics.check_invalid_state():
        self._physics.data.warning[warning_enum].number = 1
    # Existing warnings should not raise an exception.
    with self._physics.check_invalid_state():
      pass
    self._physics.reset()
    with self._physics.check_invalid_state():
      pass

  @parameterized.parameters(float('inf'), float('nan'), 1e15)
  def testBadQpos(self, bad_value):
    with self._physics.reset_context():
      self._physics.data.qpos[0] = bad_value
    with self.assertRaises(control.PhysicsError):
      with self._physics.check_invalid_state():
        mjlib.mj_checkPos(self._physics.model.ptr, self._physics.data.ptr)
    self._physics.reset()
    with self._physics.check_invalid_state():
      mjlib.mj_checkPos(self._physics.model.ptr, self._physics.data.ptr)

  def testNanControl(self):
    with self._physics.reset_context():
      pass

    # Apply the controls.
    with self.assertRaisesWithLiteralMatch(
        control.PhysicsError,
        engine._INVALID_PHYSICS_STATE.format(warning_names='mjWARN_BADCTRL')):
      with self._physics.check_invalid_state():
        self._physics.data.ctrl[0] = float('nan')
        self._physics.step()

  def testSuppressPhysicsError(self):
    bad_value = float('nan')
    message = engine._INVALID_PHYSICS_STATE.format(
        warning_names='mjWARN_BADCTRL')

    def assert_physics_error():
      self._physics.data.ctrl[0] = bad_value
      with self.assertRaisesWithLiteralMatch(control.PhysicsError, message):
        self._physics.forward()

    def assert_warning():
      self._physics.data.ctrl[0] = bad_value
      with mock.patch.object(engine.logging, 'warn') as mock_warn:
        self._physics.forward()
      mock_warn.assert_called_once_with(message)

    assert_physics_error()
    with self._physics.suppress_physics_errors():
      assert_warning()
      with self._physics.suppress_physics_errors():
        assert_warning()
      assert_warning()
    assert_physics_error()

  @parameterized.named_parameters(
      ('_copy', lambda x: x.copy()),
      ('_pickle_and_unpickle', lambda x: cPickle.loads(cPickle.dumps(x))),
  )
  def testCopyOrPicklePhysics(self, func):
    for _ in range(10):
      self._physics.step()
    physics2 = func(self._physics)
    self.assertNotEqual(physics2.model.ptr, self._physics.model.ptr)
    self.assertNotEqual(physics2.data.ptr, self._physics.data.ptr)
    model_attr_to_compare = ('nnames', 'njmax', 'body_pos', 'geom_quat')
    self._assert_attributes_equal(
        physics2.model, self._physics.model, model_attr_to_compare)
    data_attr_to_compare = ('time', 'energy', 'qpos', 'xpos')
    self._assert_attributes_equal(
        physics2.data, self._physics.data, data_attr_to_compare)
    for _ in range(10):
      self._physics.step()
      physics2.step()
    self._assert_attributes_equal(
        physics2.model, self._physics.model, model_attr_to_compare)
    self._assert_attributes_equal(
        physics2.data, self._physics.data, data_attr_to_compare)

  def testCopyDataOnly(self):
    physics2 = self._physics.copy(share_model=True)
    self.assertEqual(physics2.model.ptr, self._physics.model.ptr)
    self.assertNotEqual(physics2.data.ptr, self._physics.data.ptr)

  def testForwardDynamicsUpdatedAfterReset(self):
    gravity = -9.81
    self._physics.model.opt.gravity[2] = gravity
    with self._physics.reset_context():
      pass
    self.assertAlmostEqual(
        self._physics.named.data.sensordata['accelerometer'][2], -gravity)

  def testActuationNotAppliedInAfterReset(self):
    self._physics.data.ctrl[0] = 1.
    self._physics.after_reset()  # Calls `forward()` with actuation disabled.
    self.assertEqual(self._physics.data.actuator_force[0], 0.)
    self._physics.forward()  # Call `forward` directly with actuation enabled.
    self.assertEqual(self._physics.data.actuator_force[0], 1.)

  def testActionSpec(self):
    xml = """
    <mujoco>
      <worldbody>
        <body>
          <geom type="sphere" size="0.1"/>
          <joint type="hinge" name="hinge"/>
        </body>
      </worldbody>
      <actuator>
        <motor joint="hinge" ctrllimited="false"/>
        <motor joint="hinge" ctrllimited="true" ctrlrange="-1 2"/>
      </actuator>
    </mujoco>
    """
    physics = engine.Physics.from_xml_string(xml)
    spec = engine.action_spec(physics)
    self.assertEqual(np.float, spec.dtype)
    np.testing.assert_array_equal(spec.minimum, [-np.inf, -1.0])
    np.testing.assert_array_equal(spec.maximum, [np.inf, 2.0])

  def _check_valid_rotation_matrix(self, data_field_name):
    name = 'foo'
    quat = '0.5 0.7 0 0'  # Not normalized.
    xml_string = """
    <mujoco>
      <worldbody>
        <body name='{name}' quat='{quat}'/>
        <camera name='{name}' quat='{quat}'/>
        <geom name='{name}' quat='{quat}' size='0.1'/>
        <site name='{name}' quat='{quat}' size='0.1'/>
      </worldbody>
    </mujoco>
    """.format(name=name, quat=quat)
    physics = engine.Physics.from_xml_string(xml_string)
    rotation_matrix = getattr(physics.named.data, data_field_name)[name]
    self.assertAlmostEqual(np.linalg.det(rotation_matrix.reshape(3, 3)), 1)

  @parameterized.parameters(['xmat', 'geom_xmat', 'site_xmat'])
  def testValidRotationMatrixIfQuatNotNormalizedInXML(self, field_name):
    self._check_valid_rotation_matrix(field_name)

  # TODO(b/123918714): Update this once the bug has been fixed in MuJoCo.
  @unittest.expectedFailure
  def testValidCameraRotationMatrixIfQuatNotNormalizedInXML(self):
    self._check_valid_rotation_matrix('cam_xmat')

if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for distributions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.composer.variation import distributions
import numpy as np
from six.moves import range

RANDOM_SEED = 123
NUM_ITERATIONS = 100


def _make_random_state():
  return np.random.RandomState(RANDOM_SEED)


class DistributionsTest(parameterized.TestCase):

  def setUp(self):
    super(DistributionsTest, self).setUp()
    self._variation_random_state = _make_random_state()
    self._np_random_state = _make_random_state()

  def testUniform(self):
    lower, upper = [2, 3, 4], [5, 6, 7]
    variation = distributions.Uniform(low=lower, high=upper)
    for _ in range(NUM_ITERATIONS):
      np.testing.assert_array_equal(
          variation(random_state=self._variation_random_state),
          self._np_random_state.uniform(lower, upper))

  def testUniformChoice(self):
    choices = ['apple', 'banana', 'cherry']
    variation = distributions.UniformChoice(choices)
    for _ in range(NUM_ITERATIONS):
      self.assertEqual(
          variation(random_state=self._variation_random_state),
          self._np_random_state.choice(choices))

  def testUniformPointOnSphere(self):
    variation = distributions.UniformPointOnSphere()
    samples = []
    for _ in range(NUM_ITERATIONS):
      sample = variation(random_state=self._variation_random_state)
      self.assertEqual(sample.size, 3)
      np.testing.assert_approx_equal(np.linalg.norm(sample), 1.0)
      samples.append(sample)
    # Make sure that none of the samples are the same.
    self.assertLen(
        set(np.reshape(np.asarray(samples), -1)), 3 * NUM_ITERATIONS)

  def testNormal(self):
    loc, scale = 1, 2
    variation = distributions.Normal(loc=loc, scale=scale)
    for _ in range(NUM_ITERATIONS):
      self.assertEqual(
          variation(random_state=self._variation_random_state),
          self._np_random_state.normal(loc, scale))

  def testExponential(self):
    scale = 3
    variation = distributions.Exponential(scale=scale)
    for _ in range(NUM_ITERATIONS):
      self.assertEqual(
          variation(random_state=self._variation_random_state),
          self._np_random_state.exponential(scale))

  def testPoisson(self):
    lam = 4
    variation = distributions.Poisson(lam=lam)
    for _ in range(NUM_ITERATIONS):
      self.assertEqual(
          variation(random_state=self._variation_random_state),
          self._np_random_state.poisson(lam))

  @parameterized.parameters(0, 10)
  def testBiasedRandomWalk(self, timescale):
    stdev = 1.
    variation = distributions.BiasedRandomWalk(stdev=stdev, timescale=timescale)
    sequence = [variation(random_state=self._variation_random_state)
                for _ in range(int(max(timescale, 1)*NUM_ITERATIONS*1000))]
    self.assertAlmostEqual(np.mean(sequence), 0., delta=0.01)
    self.assertAlmostEqual(np.std(sequence), stdev, delta=0.01)

  @parameterized.parameters(
      dict(arg_name='stdev', template=distributions._NEGATIVE_STDEV),
      dict(arg_name='timescale', template=distributions._NEGATIVE_TIMESCALE))
  def testBiasedRandomWalkExceptions(self, arg_name, template):
    bad_value = -1.
    with self.assertRaisesWithLiteralMatch(
        ValueError, template.format(bad_value)):
      _ = distributions.BiasedRandomWalk(**{arg_name: bad_value})

if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tests for the keyboard module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.viewer import util
import mock
import numpy as np
from six.moves import range


class QuietSetTest(absltest.TestCase):

  def test_add_listeners(self):
    subject = util.QuietSet()
    listeners = [object() for _ in range(5)]
    for listener in listeners:
      subject += listener
    self.assertLen(subject, 5)

  def test_add_collection_of_listeners(self):
    subject = util.QuietSet()
    subject += [object() for _ in range(5)]
    self.assertLen(subject, 5)

  def test_add_collection_and_individual_listeners(self):
    subject = util.QuietSet()
    subject += object()
    subject += [object() for _ in range(5)]
    subject += object()
    self.assertLen(subject, 7)

  def test_add_duplicate_listeners(self):
    subject = util.QuietSet()
    listener = object()
    subject += listener
    self.assertLen(subject, 1)
    subject += listener
    self.assertLen(subject, 1)

  def test_remove_listeners(self):
    subject = util.QuietSet()
    listeners = [object() for _ in range(3)]
    for listener in listeners:
      subject += listener

    subject -= listeners[1]
    self.assertLen(subject, 2)

  def test_remove_unregistered_listener(self):
    subject = util.QuietSet()
    listeners = [object() for _ in range(3)]
    for listener in listeners:
      subject += listener

    subject -= object()
    self.assertLen(subject, 3)


class ToIterableTest(parameterized.TestCase):

  def test_scalars_converted_to_iterables(self):
    original_value = 3

    result = util.to_iterable(original_value)
    self.assertIsInstance(result, collections.Iterable)
    self.assertLen(result, 1)
    self.assertEqual(original_value, result[0])

  def test_strings_wrappe_by_list(self):
    original_value = 'test_string'

    result = util.to_iterable(original_value)
    self.assertIsInstance(result, collections.Iterable)
    self.assertLen(result, 1)
    self.assertEqual(original_value, result[0])

  @parameterized.named_parameters(
      ('list', [1, 2, 3]),
      ('set', set([1, 2, 3])),
      ('dict', {'1': 2, '3': 4, '5': 6})
  )
  def test_iterables_remain_unaffected(self, original_value):
    result = util.to_iterable(original_value)
    self.assertEqual(result, original_value)


class InterleaveTest(absltest.TestCase):

  def test_equal_sized_iterables(self):
    a = [1, 2, 3]
    b = [4, 5, 6]
    c = [i for i in util.interleave(a, b)]
    np.testing.assert_array_equal([1, 4, 2, 5, 3, 6], c)

  def test_iteration_ends_when_smaller_iterable_runs_out_of_elements(self):
    a = [1, 2, 3]
    b = [4, 5, 6, 7, 8]
    c = [i for i in util.interleave(a, b)]
    np.testing.assert_array_equal([1, 4, 2, 5, 3, 6], c)


class TimeMultiplierTests(absltest.TestCase):

  def setUp(self):
    super(TimeMultiplierTests, self).setUp()
    self.factor = util.TimeMultiplier(initial_time_multiplier=1.0)

  def custom_initial_factor(self):
    initial_value = 5.0
    factor = util.TimeMultiplier(initial_time_multiplier=initial_value)
    self.assertEqual(initial_value, factor.get())

  def initial_factor_clamped_to_valid_value_range(self):
    too_large_multiplier = util._MAX_TIME_MULTIPLIER + 1.
    too_small_multiplier = util._MIN_TIME_MULTIPLIER - 1.

    factor = util.TimeMultiplier(initial_time_multiplier=too_large_multiplier)
    self.assertEqual(util._MAX_TIME_MULTIPLIER, factor.get())

    factor = util.TimeMultiplier(initial_time_multiplier=too_small_multiplier)
    self.assertEqual(util._MIN_TIME_MULTIPLIER, factor.get())

  def test_increase(self):
    self.factor.increase()
    self.assertEqual(self.factor._real_time_multiplier, 2.0)

  def test_increase_limit(self):
    self.factor._real_time_multiplier = util._MAX_TIME_MULTIPLIER
    self.factor.increase()
    self.assertEqual(util._MAX_TIME_MULTIPLIER, self.factor.get())

  def test_decrease(self):
    self.factor.decrease()
    self.factor.decrease()
    self.assertEqual(self.factor._real_time_multiplier, 0.25)

  def test_decrease_limit(self):
    self.factor._real_time_multiplier = util._MIN_TIME_MULTIPLIER
    self.factor.decrease()
    self.assertEqual(util._MIN_TIME_MULTIPLIER, self.factor.get())

  def test_stringify_when_greater_than_one(self):
    self.assertEqual('1', str(self.factor))
    self.factor.increase()
    self.assertEqual('2', str(self.factor))
    self.factor.increase()
    self.assertEqual('4', str(self.factor))

  def test_stringify_when_less_than_one(self):
    self.assertEqual('1', str(self.factor))
    self.factor.decrease()
    self.assertEqual('1/2', str(self.factor))
    self.factor.decrease()
    self.assertEqual('1/4', str(self.factor))


class IntegratorTests(absltest.TestCase):

  def setUp(self):
    super(IntegratorTests, self).setUp()
    self.integration_step = 1
    self.integrator = util.Integrator(self.integration_step)
    self.integrator._sampling_timestamp = 0.0

  def test_initial_value(self):
    self.assertEqual(0, self.integrator.value)

  def test_integration_step(self):
    with mock.patch(util.__name__ + '.time') as time_mock:
      time_mock.time.return_value = self.integration_step
      self.integrator.value = 1
    self.assertEqual(1, self.integrator.value)

  def test_averaging(self):
    with mock.patch(util.__name__ + '.time') as time_mock:
      time_mock.time.return_value = 0
      self.integrator.value = 1
      self.integrator.value = 1
      self.integrator.value = 1
      time_mock.time.return_value = self.integration_step
      self.integrator.value = 1
    self.assertEqual(1, self.integrator.value)


class AtomicActionTests(absltest.TestCase):

  def setUp(self):
    super(AtomicActionTests, self).setUp()
    self.callback = mock.MagicMock()
    self.action = util.AtomicAction(self.callback)

  def test_starting_and_ending_one_action(self):
    self.action.begin(1)
    self.assertEqual(1, self.action.watermark)
    self.callback.assert_called_once_with(1)

    self.callback.reset_mock()

    self.action.end(1)
    self.assertEqual(None, self.action.watermark)
    self.callback.assert_called_once_with(None)

  def test_trying_to_interrupt_with_another_action(self):
    self.action.begin(1)
    self.assertEqual(1, self.action.watermark)
    self.callback.assert_called_once_with(1)

    self.callback.reset_mock()

    self.action.begin(2)
    self.assertEqual(1, self.action.watermark)
    self.assertEqual(0, self.callback.call_count)

  def test_trying_to_end_another_action(self):
    self.action.begin(1)
    self.callback.reset_mock()

    self.action.end(2)
    self.assertEqual(1, self.action.watermark)
    self.assertEqual(0, self.callback.call_count)


class ObservableFlagTest(absltest.TestCase):

  def test_update_each_added_listener(self):
    listener = mock.MagicMock(spec=object)

    subject = util.ObservableFlag(True)

    subject += listener
    listener.assert_called_once_with(True)

  def test_update_listeners_on_toggle(self):
    listeners = [mock.MagicMock(spec=object) for _ in range(10)]

    subject = util.ObservableFlag(True)
    subject += listeners

    for listener in listeners:
      listener.reset_mock()
    subject.toggle()
    for listener in listeners:
      listener.assert_called_once_with(False)


class TimerTest(absltest.TestCase):

  def setUp(self):
    super(TimerTest, self).setUp()
    self.timer = util.Timer()

  def test_time_elapsed(self):
    with mock.patch(util.__name__ + '.time') as time_mock:
      time_mock.time.return_value = 1
      self.timer.tick()
      time_mock.time.return_value = 2
      self.assertEqual(1, self.timer.tick())

  def test_time_measurement(self):
    with mock.patch(util.__name__ + '.time') as time_mock:
      time_mock.time.return_value = 1
      with self.timer.measure_time():
        time_mock.time.return_value = 4
      self.assertEqual(3, self.timer.measured_time)


class ErrorLoggerTest(absltest.TestCase):

  def setUp(self):
    super(ErrorLoggerTest, self).setUp()
    self.callback = mock.MagicMock()
    self.logger = util.ErrorLogger([self.callback])

  def test_no_errors_found_on_initialization(self):
    self.assertFalse(self.logger.errors_found)

  def test_no_error_caught(self):
    with self.logger:
      pass
    self.assertFalse(self.logger.errors_found)

  def test_error_caught(self):
    with self.logger:
      raise Exception('error message')
    self.assertTrue(self.logger.errors_found)

  def test_notifying_callbacks(self):
    error_message = 'error message'
    with self.logger:
      raise Exception(error_message)
    self.callback.assert_called_once_with(error_message)


class NullErrorLoggerTest(absltest.TestCase):

  def setUp(self):
    super(NullErrorLoggerTest, self).setUp()
    self.logger = util.NullErrorLogger()

  def test_thrown_errors_are_not_being_intercepted(self):
    with self.assertRaises(Exception):
      with self.logger:
        raise Exception()

  def test_errors_found_always_returns_false(self):
    self.assertFalse(self.logger.errors_found)


if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for `dm_control.mjcf.copier`."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

# Internal dependencies.
from absl.testing import absltest
from dm_control import mjcf
from dm_control.mjcf import parser
import numpy as np
import six

_ASSETS_DIR = os.path.join(os.path.dirname(__file__), 'test_assets')
_TEST_MODEL_XML = os.path.join(_ASSETS_DIR, 'test_model.xml')
_MODEL_WITH_ASSETS_XML = os.path.join(_ASSETS_DIR, 'model_with_assets.xml')


class CopierTest(absltest.TestCase):

  def testSimpleCopy(self):
    mjcf_model = parser.from_path(_TEST_MODEL_XML)
    mixin = mjcf.RootElement(model='test_mixin')
    mixin.compiler.boundmass = 1
    mjcf_model.include_copy(mixin)
    self.assertEqual(mjcf_model.model, 'test')  # Model name should not change
    self.assertEqual(mjcf_model.compiler.boundmass, mixin.compiler.boundmass)
    mixin.compiler.boundinertia = 2
    mjcf_model.include_copy(mixin)
    self.assertEqual(mjcf_model.compiler.boundinertia,
                     mixin.compiler.boundinertia)
    mixin.compiler.boundinertia = 1
    with six.assertRaisesRegex(self, ValueError, 'Conflicting values'):
      mjcf_model.include_copy(mixin)
    mixin.worldbody.add('body', name='b_0', pos=[0, 1, 2])
    mjcf_model.include_copy(mixin, override_attributes=True)
    self.assertEqual(mjcf_model.compiler.boundmass, mixin.compiler.boundmass)
    self.assertEqual(mjcf_model.compiler.boundinertia,
                     mixin.compiler.boundinertia)
    np.testing.assert_array_equal(mjcf_model.worldbody.body['b_0'].pos,
                                  [0, 1, 2])

  def testCopyingWithReference(self):
    sensor_mixin = mjcf.RootElement('sensor_mixin')
    touch_site = sensor_mixin.worldbody.add('site', name='touch_site')
    sensor_mixin.sensor.add('touch', name='touch_sensor', site=touch_site)

    mjcf_model = mjcf.RootElement('model')
    mjcf_model.include_copy(sensor_mixin)

    # Copied reference should be updated to the copied site.
    self.assertIs(mjcf_model.find('sensor', 'touch_sensor').site,
                  mjcf_model.find('site', 'touch_site'))

  def testCopyingWithAssets(self):
    mjcf_model = parser.from_path(_MODEL_WITH_ASSETS_XML)
    copied = mjcf.RootElement()
    copied.include_copy(mjcf_model)

    original_assets = (mjcf_model.find_all('mesh')
                       + mjcf_model.find_all('texture')
                       + mjcf_model.find_all('hfield'))
    copied_assets = (copied.find_all('mesh')
                     + copied.find_all('texture')
                     + copied.find_all('hfield'))

    self.assertLen(copied_assets, len(original_assets))
    for original_asset, copied_asset in zip(original_assets, copied_assets):
      self.assertIs(copied_asset.file, original_asset.file)

if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Helper object for keeping track of new elements created when copying MJCF."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control.mjcf import constants


class Copier(object):
  """Helper for keeping track of new elements created when copying MJCF."""

  def __init__(self, source):
    if source._attachments:  # pylint: disable=protected-access
      raise NotImplementedError('Cannot copy from elements with attachments')
    self._source = source

  def copy_into(self, destination, override_attributes=False):
    """Copies this copier's element into a destination MJCF element."""
    newly_created_elements = {}
    destination._check_valid_attachment(self._source)  # pylint: disable=protected-access
    if override_attributes:
      destination.set_attributes(**self._source.get_attributes())
    else:
      destination._sync_attributes(self._source, copying=True)  # pylint: disable=protected-access
    for source_child in self._source.all_children():
      dest_child = None
      # First, if source_child has an identifier, we look for an existing child
      # element of self with the same identifier to override.
      if source_child.spec.identifier and override_attributes:
        identifier_attr = source_child.spec.identifier
        if identifier_attr == constants.CLASS:
          identifier_attr = constants.DCLASS
        identifier = getattr(source_child, identifier_attr)
        if identifier:
          dest_child = destination.find(source_child.spec.namespace, identifier)
        if dest_child is not None and dest_child.parent is not destination:
          raise ValueError(
              '<{}> with identifier {!r} is already a child of another element'
              .format(source_child.spec.namespace, identifier))
      # Next, we cover the case where either the child is not a repeated element
      # or if source_child has an identifier attribute but it isn't set.
      if not source_child.spec.repeated and dest_child is None:
        dest_child = destination.get_children(source_child.tag)

      # Add a new element if dest_child doesn't exist, either because it is
      # supposed to be a repeated child, or because it's an uncreated on-demand.
      if dest_child is None:
        dest_child = destination.add(
            source_child.tag, **source_child.get_attributes())
        newly_created_elements[source_child] = dest_child
        override_child_attributes = True
      else:
        override_child_attributes = override_attributes

      # Finally, copy attributes into dest_child.
      child_copier = Copier(source_child)
      newly_created_elements.update(
          child_copier.copy_into(dest_child, override_child_attributes))
    return newly_created_elements



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Utility functions that operate on MJCF elements."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


def get_freejoint(element):
  """Retrieves the free joint of a body. Returns `None` if there isn't one."""
  if element.tag != 'body':
    return None
  elif hasattr(element, 'freejoint') and element.freejoint is not None:
    return element.freejoint
  else:
    joints = element.find_all('joint', immediate_children_only=True)
    for joint in joints:
      if joint.type == 'free':
        return joint
    return None


def get_attachment_frame(mjcf_model):
  return mjcf_model.parent_model.find('attachment_frame', mjcf_model.model)


def get_frame_freejoint(mjcf_model):
  frame = get_attachment_frame(mjcf_model)
  return get_freejoint(frame)


def get_frame_joints(mjcf_model):
  """Retrieves all joints belonging to the attachment frame of an MJCF model."""
  frame = get_attachment_frame(mjcf_model)
  if frame:
    return frame.find_all('joint', immediate_children_only=True)
  else:
    return None



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Decorators for Entity methods returning elements and observables."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc
import threading


class cached_property(property):  # pylint: disable=invalid-name
  """A property that is evaluated only once per object instance."""

  def __init__(self, func, doc=None):
    super(cached_property, self).__init__(fget=func, doc=doc)
    self.lock = threading.RLock()

  def __get__(self, obj, cls):
    if obj is None:
      return self
    name = self.fget.__name__
    obj_dict = obj.__dict__
    try:
      # Try returning a precomputed value without locking first.
      # Profiling shows that the lock takes up a non-trivial amount of time.
      return obj_dict[name]
    except KeyError:
      # The value hasn't been computed, now we have to lock.
      with self.lock:
        try:
          # Check again whether another thread has already computed the value.
          return obj_dict[name]
        except KeyError:
          # Otherwise call the function, cache the result, and return it
          return obj_dict.setdefault(name, self.fget(obj))


# A decorator for base.Observables methods returning an observable. This
# decorator should be used by abstract base classes to indicate sub-classes need
# to implement a corresponding @observavble annotated method.
abstract_observable = abc.abstractproperty  # pylint: disable=invalid-name


class observable(cached_property):  # pylint: disable=invalid-name
  """A decorator for base.Observables methods returning an observable.

  The body of the decorated function is evaluated at Entity construction time
  and the observable is cached.
  """
  pass



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""An initializer that places props at various poses."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys

from dm_control import composer
from dm_control import mjcf
from dm_control.composer import variation
from dm_control.composer.initializers import utils
from dm_control.composer.variation import rotations
from dm_control.rl import control
import numpy as np
import six
from six.moves import range


# Absolute velocity threshold for a prop joint to be considered settled.
_SETTLE_QVEL_TOL = 1e-3
# Absolute acceleration threshold for a prop joint to be considered settled.
_SETTLE_QACC_TOL = 1e-2

_REJECTION_SAMPLING_FAILED = '\n'.join([
    'Failed to find a non-colliding pose for prop {model_name!r} within '
    '{max_attempts} attempts.',
    'You may be able to avoid this error by:',
    '1. Sampling from a broader distribution over positions and/or quaternions',
    '2. Increasing `max_attempts_per_prop`',
    '3. Disabling collision detection by setting `ignore_collisions=False`'])


class PropPlacer(composer.Initializer):
  """An initializer that places props at various positions and orientations."""

  def __init__(self, props, position, quaternion=rotations.IDENTITY_QUATERNION,
               ignore_collisions=False, max_attempts_per_prop=20,
               settle_physics=False, max_settle_physics_time=2.):
    """Initializes this PropPlacer.

    Args:
      props: A sequence of `composer.Entity` instances representing props.
      position: A single fixed Cartesian position, or a `composer.Variation`
        object that generates Cartesian positions. If a fixed sequence of
        positions for multiple props is desired, use
        `variation.deterministic.Sequence`.
      quaternion: (optional) A single fixed unit quaternion, or a
        `Variation` object that generates unit quaternions. If a fixed
        sequence of quaternions for multiple props is desired, use
        `variation.deterministic.Sequence`.
      ignore_collisions: (optional) If True, ignore collisions between props,
        i.e. do not run rejection sampling.
      max_attempts_per_prop: The maximum number of rejection sampling attempts
        per prop. If a non-colliding pose cannot be found before this limit is
        reached, a `RuntimeError` will be raised.
      settle_physics: (optional) If True, the physics simulation will be
        advanced for a few steps to allow the prop positions to settle.
      max_settle_physics_time: (optional) When `settle_physics` is True, upper
        bound on time (in seconds) the physics simulation is advanced.
    """
    super(PropPlacer, self).__init__()
    self._props = props
    self._prop_joints = []
    for prop in props:
      freejoint = mjcf.get_frame_freejoint(prop.mjcf_model)
      if freejoint is not None:
        self._prop_joints.append(freejoint)
      self._prop_joints.extend(prop.mjcf_model.find_all('joint'))
    self._position = position
    self._quaternion = quaternion
    self._ignore_collisions = ignore_collisions
    self._max_attempts_per_prop = max_attempts_per_prop
    self._settle_physics = settle_physics
    self._max_settle_physics_time = max_settle_physics_time

  def _has_collisions_with_prop(self, physics, prop):
    prop_geom_ids = physics.bind(prop.mjcf_model.find_all('geom')).element_id
    contact = physics.data.contact
    involves_prop = (np.in1d(contact.geom1, prop_geom_ids) |
                     np.in1d(contact.geom2, prop_geom_ids))
    # Ignore contacts with positive distances (i.e. not actually touching).
    touching = contact.dist <= 0
    return np.any(involves_prop & touching)

  def _disable_and_cache_contact_parameters(self, physics, props):
    cached_contact_params = {}
    for prop in props:
      geoms = prop.mjcf_model.find_all('geom')
      param_list = []
      for geom in geoms:
        bound_geom = physics.bind(geom)
        param_list.append((bound_geom.contype, bound_geom.conaffinity))
        bound_geom.contype = 0
        bound_geom.conaffinity = 0
      cached_contact_params[prop] = param_list
    return cached_contact_params

  def _restore_contact_parameters(self, physics, prop, cached_contact_params):
    geoms = prop.mjcf_model.find_all('geom')
    param_list = cached_contact_params[prop]
    for i, geom in enumerate(geoms):
      contype, conaffinity = param_list[i]
      bound_geom = physics.bind(geom)
      bound_geom.contype = contype
      bound_geom.conaffinity = conaffinity

  def __call__(self, physics, random_state, ignore_contacts_with_entities=None):
    """Sets initial prop poses.

    Args:
      physics: An `mjcf.Physics` instance.
      random_state: a `np.random.RandomState` instance.
      ignore_contacts_with_entities: a list of `composer.Entity` instances
        to ignore when detecting collisions. This can be used to ignore props
        that are not being placed by this initializer, but are known to be
        colliding in the current state of the simulation (for example if they
        are going to be placed by a different initializer that will be called
        subsequently).

    Raises:
      RuntimeError: If `ignore_collisions == False` and a non-colliding prop
        pose could not be found within `max_attempts_per_prop`.
    """
    if ignore_contacts_with_entities is None:
      ignore_contacts_with_entities = []
    # Temporarily disable contacts for all geoms that belong to props which
    # haven't yet been placed in order to free up space in the contact buffer.
    cached_contact_params = self._disable_and_cache_contact_parameters(
        physics, self._props + ignore_contacts_with_entities)

    try:
      physics.forward()
    except control.PhysicsError as e:
      _, _, tb = sys.exc_info()
      message = 'Despite disabling contact for all props in this initializer, '
      '`physics.forward()` resulted in a `PhysicsError`: {}'.format(e)
      new_exc = control.PhysicsError(message)
      six.reraise(control.PhysicsError, new_exc, tb)

    for prop in self._props:

      # Restore the original contact parameters for all geoms in the prop we're
      # about to place, so that we can detect if the new pose results in
      # collisions.
      self._restore_contact_parameters(physics, prop, cached_contact_params)

      success = False
      initial_position, initial_quaternion = prop.get_pose(physics)
      next_position, next_quaternion = initial_position, initial_quaternion
      for _ in range(self._max_attempts_per_prop):
        next_position = variation.evaluate(self._position,
                                           initial_value=initial_position,
                                           current_value=next_position,
                                           random_state=random_state)
        next_quaternion = variation.evaluate(self._quaternion,
                                             initial_value=initial_quaternion,
                                             current_value=next_quaternion,
                                             random_state=random_state)
        prop.set_pose(physics, next_position, next_quaternion)
        try:
          # If this pose results in collisions then there's a chance we'll
          # encounter a PhysicsError error here due to a full contact buffer,
          # in which case reject this pose and sample another.
          physics.forward()
        except control.PhysicsError:
          continue
        if (self._ignore_collisions
            or not self._has_collisions_with_prop(physics, prop)):
          success = True
          break

      if not success:
        raise RuntimeError(_REJECTION_SAMPLING_FAILED.format(
            model_name=prop.mjcf_model.model,
            max_attempts=self._max_attempts_per_prop))

    for prop in ignore_contacts_with_entities:
      self._restore_contact_parameters(physics, prop, cached_contact_params)

    if self._settle_physics:
      original_time = physics.data.time
      props_isolator = utils.JointStaticIsolator(physics, self._prop_joints)
      prop_joints_mj = physics.bind(self._prop_joints)
      while physics.data.time - original_time < self._max_settle_physics_time:
        with props_isolator:
          physics.step()
        if (np.max(np.abs(prop_joints_mj.qvel)) < _SETTLE_QVEL_TOL and
            np.max(np.abs(prop_joints_mj.qacc)) < _SETTLE_QACC_TOL):
          break
      physics.data.time = original_time
      # TODO(b/120221805): We ought to raise an exception if settling fails.



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tests for props.target_sphere."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.
from absl.testing import absltest
from dm_control import composer

from dm_control.entities.props import primitive
from dm_control.locomotion.arenas import floors
from dm_control.locomotion.props import target_sphere


class TargetSphereTest(absltest.TestCase):

  def testActivation(self):
    target_radius = 0.6
    prop_radius = 0.1
    target_height = 1

    arena = floors.Floor()
    target = target_sphere.TargetSphere(radius=target_radius,
                                        height_above_ground=target_height)
    prop = primitive.Primitive(geom_type='sphere', size=[prop_radius])
    arena.attach(target)
    arena.add_free_entity(prop)

    task = composer.NullTask(arena)
    task.initialize_episode = (
        lambda physics, random_state: prop.set_pose(physics, [0, 0, 2]))

    env = composer.Environment(task)
    env.reset()

    max_activated_height = target_height + target_radius + prop_radius

    while env.physics.bind(prop.geom).xpos[2] > max_activated_height:
      self.assertFalse(target.activated)
      self.assertEqual(env.physics.bind(target.material).rgba[-1], 1)
      env.step([])

    while env.physics.bind(prop.geom).xpos[2] > 0.2:
      self.assertTrue(target.activated)
      self.assertEqual(env.physics.bind(target.material).rgba[-1], 0)
      env.step([])

    # Target should be reset when the environment is reset.
    env.reset()
    self.assertFalse(target.activated)
    self.assertEqual(env.physics.bind(target.material).rgba[-1], 1)


if __name__ == '__main__':
  absltest.main()
